//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      AlarmSettingDialog.cpp
//

#include "AlarmSettingDialog.hpp"
#include "pclock.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"

#define ALARMSETTINGDIALOG_LABEL_SOFTKEY1   "Set"
#define ALARMSETTINGDIALOG_LABEL_SOFTKEY2   "Cancel"
#define ALARMSETTINGDIALOG_LABEL_SOFTKEY3   "Select"
#define ALARMSETTINGDIALOG_ERRORDIALOG_HEIGHT       220

AlarmSettingDialog::AlarmSettingDialog(SFXDateDurationConstRef timeDifference) static_throws : SFRTitleDialog(SGRSoftkeyToolbox::GetWindowPosition(), "Alarm setup"), _onOff(null), _subject(null), _date(null), _hour(null), _minute(null), _dateDialog(null), _timeDifference(timeDifference)
{
    SFXRectangle world(GetContentWorld());
    SGRSoftkeyWindowPtr softkey;
    SFXWideString comboText[] = {"OFF", "ON"};
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    pclock::AlarmInformationConst alarmInfo(app->GetAlarm());
    Bool alarmOn;
    SFXWideString subjectString, dateString;

    SGRSoftkeyWindow::Initialize(this);

    // Coordinates calculation
    world.Deflate(10, 10, 10, 10);
    _onOffTextRect.Set(world);
    _onOffTextRect.DivHeight(8);
    _onOffControlRect.Set(_onOffTextRect);
    _onOffControlRect.Offset(0, _onOffTextRect.GetHeight());
    _subjectTextRect.Set(_onOffControlRect);
    _subjectTextRect.Offset(0, _onOffControlRect.GetHeight());
    _subjectControlRect.Set(_subjectTextRect);
    _subjectControlRect.Offset(0, _subjectTextRect.GetHeight());
    _dateTextRect.Set(_subjectControlRect);
    _dateTextRect.Offset(0, _subjectControlRect.GetHeight());
    _dateControlRect.Set(_dateTextRect);
    _dateControlRect.Offset(0, _dateTextRect.GetHeight());
    _timeTextRect.Set(_dateControlRect);
    _timeTextRect.Offset(0, _dateTextRect.GetHeight());
    _hourControlRect.Set(_timeTextRect);
    _hourControlRect.Offset(0, _timeTextRect.GetHeight());
    _onOffControlRect.SetWidth(120);
    _onOffControlRect.Offset(10, 0);
    _subjectControlRect.Deflate(10, 0, 30, 0);
    _dateControlRect.SetWidth(120);
    _dateControlRect.Offset(10, 0);
    _hourControlRect.SetWidth(120);
    _hourControlRect.Offset(10, 0);
    _hourControlRect.DivWidth(4);
    _hourTextRect.Set(_hourControlRect);
    _hourTextRect.Offset(_hourControlRect.GetWidth(), 0);
    _minuteControlRect.Set(_hourTextRect);
    _minuteControlRect.Offset(_hourTextRect.GetWidth(), 0);
    _minuteTextRect.Set(_minuteControlRect);
    _minuteTextRect.Offset(_minuteControlRect.GetWidth(), 0);

    // The validity of the alarm is checked
    if (alarmInfo._date <= SFXDate::CurrentUTCDate()) {
        alarmOn = false;
        subjectString.Clear();
        _alarmLocalDate = SFXDate::CurrentUTCDate() + _timeDifference;
        _alarmLocalDate.Set(_alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay(), _alarmLocalDate.GetHour(), 0, 0);
        _alarmLocalDate.AddHour(1);
        dateString = SFXWideString::Format("%04hd/%02hd/%02hd", _alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay());
    }
    else if (!app->IsAlarmActive()) {
        alarmOn = false;
        subjectString.Set(alarmInfo._subject);
        _alarmLocalDate = alarmInfo._date + _timeDifference;
        dateString = SFXWideString::Format("%04hd/%02hd/%02hd", _alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay());
    }
    else {
        alarmOn = true;
        subjectString.Set(alarmInfo._subject);
        _alarmLocalDate = alarmInfo._date + _timeDifference;
        dateString = SFXWideString::Format("%04hd/%02hd/%02hd", _alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay());
    }

    // Responder generation
    if (static_try()) {
        _onOff = ::new SFRComboboxControl(this, _onOffControlRect, comboText, lengthof(comboText));
        if (!_onOff) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_onOff);
        }
    }
    if (!static_try()) {
        ::delete _onOff;
        _onOff = null;
    }
    else {
        _onOff->SetValue(alarmOn ? 1 : 0);
    }
    if (static_try()) {
        _subject = ::new SFREditboxControl(this, _subjectControlRect, subjectString);
        if (!_subject) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_subject);
        }
        if (!static_try()) {
            ::delete _subject;
            _subject = null;
        }
        else {
            _subject->SetMaxSize(128);
            _subject->SetStatusEnable(alarmOn);
        }
    }
    if (static_try()) {
        _date = ::new SFRButtonControl(this, _dateControlRect, dateString);
        if (!_date) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_date);
        }
        if (!static_try()) {
            ::delete _date;
            _date = null;
        }
        else {
            _date->SetStatusEnable(alarmOn);
        }
    }
    if (static_try()) {
        _hour = ::new SGRNumberTextControl(this, _hourControlRect, 2, 0, 23, _alarmLocalDate.GetHour());
        if (!_hour) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_hour);
        }
        if (!static_try()) {
            ::delete _hour;
            _hour = null;
        }
        else {
            _hour->SetStatusEnable(alarmOn);
        }
    }
    if (static_try()) {
        _minute = ::new SGRNumberTextControl(this, _minuteControlRect, 2, 0, 59, _alarmLocalDate.GetMinute());
        if (!_minute) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_minute);
        }
        if (!static_try()) {
            ::delete _minute;
            _minute = null;
        }
        else {
            _minute->SetStatusEnable(alarmOn);
        }
    }

    // Handler registrations
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
    if (static_try()) {
        static_throw(SFRToolbox::RegisterManipulateHandler(this));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, AVK_SELECT, HANDLER_AFTER, HANDLER_FUNCTION(OnSelect)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
    }
    if (static_try()) {
        static_throw(_onOff->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnAlarmCombo)));
    }
    if (static_try()) {
        static_throw(_subject->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }
    if (static_try()) {
        static_throw(_date->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnDateButton)));
    }

    // Softkey registrations
    if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) {
        static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_1, ALARMSETTINGDIALOG_LABEL_SOFTKEY1));
        if (static_try()) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_2, ALARMSETTINGDIALOG_LABEL_SOFTKEY2));
        }
        if (static_try()) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, ALARMSETTINGDIALOG_LABEL_SOFTKEY3));
        }
        if (static_try()) {
            SetReference(softkey->Bind(this));
        }
    }

    if (_onOff) {
        _onOff->SetStatusFocus(true);
    }
}

AlarmSettingDialog::~AlarmSettingDialog(Void)
{
    SGRSoftkeyWindow::Terminate(this);
}

HANDLER_IMPLEMENT_VOIDRENDER(AlarmSettingDialog, OnContent, graphics)
{
    graphics->ClearRectangle(GetContentWorld());
    graphics->DrawText("Alarm ON/OFF", _onOffTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Subject", _subjectTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Date", _dateTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Time", _timeTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Hour", _hourTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Minute", _minuteTextRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
}

//
// Select key handler.
//
HANDLER_IMPLEMENT_BOOLVOID(AlarmSettingDialog, OnSelect)
{
    SFRControlPtr control;
    Bool result(false);

    if ((control = static_cast<SFRControlPtr>(GetFocus())) != null) {
        if (control->GetAttribute() == ATTRIBUTE_SFREDITBOXCONTROL) {
            control->SetStatusTarget(true);
#if defined TARGET_ENVIRONMENT_PHYSICAL
            result = control->Invoke(SFXEvent(SFEVT_KEY, AVK_SELECT, 0));
#else
            result = true;
#endif
        }
    }

    return result;
}

//
// Text control input end handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(AlarmSettingDialog, OnEditbox, result, control)
{
    unused(result);

    control->SetStatusTarget(false);
    return;
}

//
// Event handler of the alarm ON/OFF combo box.
//
HANDLER_IMPLEMENT_VOIDCONTROL(AlarmSettingDialog, OnAlarmCombo, result, control)
{
    unused(control);

    Bool alarmOn(false);

    if (result != SRP16_ESCAPE) {
        alarmOn = (_onOff->GetValue() == 1);
        if (_subject) {
            _subject->SetStatusEnable(alarmOn);
        }
        if (_date) {
            _date->SetStatusEnable(alarmOn);
        }
        if (_hour) {
            _hour->SetStatusEnable(alarmOn);
        }
        if (_minute) {
            _minute->SetStatusEnable(alarmOn);
        }
        if (_onOff && alarmOn) {
            _onOff->SetStatusFocus(true);
        }
    }
}

//
// Date select button event handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(AlarmSettingDialog, OnDateButton, result, control)
{
    unused(result);
    unused(control);

    if (static_try()) {
        _dateDialog = ::new DateSelectionDialog(_alarmLocalDate, _timeDifference);
        if (!_dateDialog) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_dateDialog);
        }
        if (!static_try()) {
            ::delete _dateDialog;
            _dateDialog = null;
        }
    }

    if (static_try()) {
        static_throw(_dateDialog->RegisterHandler(SREVT_WINDOW, HANDLER_AFTER, HANDLER_FUNCTION(OnDateSelectionDialog)));
    }
}

HANDLER_IMPLEMENT_VOIDEVENT(AlarmSettingDialog, OnDateSelectionDialog, event)
{
    if (event.GetP16() == SRP16_OK && _dateDialog) {
        _alarmLocalDate = _dateDialog->GetCursor();
        _dateDialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        if (_date) {
            _date->SetTitle(SFXWideString::Format("%04hd/%02hd/%02hd", _alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay()));
            InvalidateContent();
        }
    }
}

HANDLER_IMPLEMENT_VOIDEVENT(AlarmSettingDialog, OnSoftkey, event)
{
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    pclock::AlarmInformation alarmInfo;
    SFXDate localDate;
    Bool alarmOn(false), result(false);
    SFRMultiDialogPtr dialog(null);
    SFRMultiDialog::ParamRec param;

    switch (event.GetP16()) {
    case SGRSoftkeyWindow::SOFTKEY_1:
        // Set Time
        if (_onOff) {
            alarmOn = (_onOff->GetValue() == 1);
        }
        if (!alarmOn && app->IsAlarmActive()) {
            result = app->CancelAlarm();
            if (!result) {
                // It failed releasing the alarm
                param.text = "The alarm was not able to be released. ";
                param.ok = "OK";
                param.cancel = "";
                param.another = "";
                param.focus = SFRMultiDialog::FOCUS_OK;
                param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
            }
        }
        else {
            if (_subject && _hour && _minute) {
                alarmInfo._subject.Set(_subject->GetText());
                localDate.Set(_alarmLocalDate.GetYear(), _alarmLocalDate.GetMonth(), _alarmLocalDate.GetDay(), static_cast<UInt16>(_hour->GetValue()), static_cast<UInt16>(_minute->GetValue()), 0);
                alarmInfo._date = localDate - _timeDifference;
                if (alarmInfo._date <= SFXDate::CurrentUTCDate()) {
                    // It tried to set a past time
                    param.text = "A past date cannot be set. ";
                    param.ok = "OK";
                    param.cancel = "";
                    param.another = "";
                    param.focus = SFRMultiDialog::FOCUS_OK;
                    param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
                    result = false;
                }
                else {
                    result = app->SetAlarm(alarmInfo);
                    if (!result) {
                        param.text = "The alarm was not able to be set.";
                        param.ok = "OK";
                        param.cancel = "";
                        param.another = "";
                        param.focus = SFRMultiDialog::FOCUS_OK;
                        param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
                    }
                }
            }
        }
        if (!result) {
            dialog = ::new SFRMultiDialog(SGRSoftkeyToolbox::GetDialogPosition(SFRMultiDialog::GetMargin(), ALARMSETTINGDIALOG_ERRORDIALOG_HEIGHT), "G[", param);
            if (dialog) {
                static_throw(*dialog);
            }
            if (!static_try()) {
                ::delete dialog;
                dialog = null;
            }
            else {
                SGRSoftkeyToolbox::OpenStandardDialog(dialog, HANDLER_FUNCTION(OnErrorDialog));
            }
        }
        else {
            Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        }
        break;
    case SGRSoftkeyWindow::SOFTKEY_2:
        // Cancel is selected
        Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        break;
    }
}

HANDLER_IMPLEMENT_VOIDDIALOG(AlarmSettingDialog, OnErrorDialog, result, dialog)
{
    unused(result);

    SGRSoftkeyToolbox::CloseStandardDialog(dialog);
}
