//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      AstronomyPane.cpp
//

#include "AstronomyPane.hpp"
#include "ClockWindow.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"
#include "moon_res.h"
#include "pclock.hpp"

#define ASTRONOMYPANE_MOONAGERESFILE                "moon.bar"          // The moon source file for picture
#define ASTRONOMYPANE_MOONAGE_WIDTH                 60                  // The width of the moon picture
#define ASTRONOMYPANE_MOONAGE_HEIGHT                60                  // The height of teh moon picture
#define ASTRONOMYPANE_MOONAGE_TRANSPARENCYCOLOR     255, 0, 255, 0      // The transparency color of the moon picture
#define ASTRONOMYPANE_UPDATE_DELAY                  15                  // Time delayed when it is positive in update (sec)
#define ASTRONOMYPANE_BACKGROUND_COLOR              255, 255, 255, 0    // The background color

AstronomyPane::AstronomyPane(SFRResponderPtr responder, SFXRectangleConstRef rect, SFXDateDurationConstRef timeDifference, Float64 longitude, Float64 latitude, Float64 height) : SFRPane(responder, rect, SFXMargin(0, 0, 0, 0)), _moonAge(-1.0), _timeDifference(timeDifference), _longitude(longitude), _latitude(latitude), _height(height), _autoUpdate(true)
{
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_APP_RESUME, HANDLER_BEFORE, HANDLER_FUNCTION(OnAppResume)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_APP_SUSPEND, HANDLER_AFTER, HANDLER_FUNCTION(OnAppSuspend)));
    }
    return;
}

AstronomyPane::~AstronomyPane(Void)
{
    _hourTimer.Cancel();
    _infoTimer.Cancel();
    return;
}

SFXWideString AstronomyPane::GetMoonAgeString(Void) const
{
    SFXWideString str;
    SInt32 moonAge10;

    if (_moonAge >= 0.0) {
        moonAge10 = static_cast<SInt32>(floor(_moonAge * 10.0 + 0.5));
        str = SFXWideString::Format("Moon age %2d.%d", moonAge10 / 10, moonAge10 % 10);
    }
    else {  // Not calculated yet
        str = "Moon old --.-";
    }
    return str;
}

SFXWideString AstronomyPane::GetSunRiseString(Void) const
{
    SFXWideString str;
    AstronomicalMath::Date date;
    SFXDate localDate;

    if (_moonAge < 0.0) {
        // Not calculated yet
        str = "Sun Rise --:--";
    }
    else {
        if (_sun.GetRiseDateSize() == 0) {
            str = "Sun Rise --:--";
        }
        else {
            date = _sun.GetRiseDate(0);
            localDate = date.GetSFXDate() + _timeDifference + SFXDateDuration(30);
            str = SFXWideString::Format("Sun Rise %02hd:%02hd", localDate.GetHour(), localDate.GetMinute());
        }
    }
    return str;
}

SFXWideString AstronomyPane::GetSunSetString(Void) const
{
    SFXWideString str;
    AstronomicalMath::Date date;
    SFXDate localDate;

    if (_moonAge < 0.0) {
        // Not calculated yet
        str = "Sun Set  --:--";
    }
    else {
        if (_sun.GetSetDateSize() == 0) {
            str = "Sun Set  --:--";
        }
        else {
            date = _sun.GetSetDate(0);
            localDate = date.GetSFXDate() + _timeDifference + SFXDateDuration(30);
            str = SFXWideString::Format("Sun Set  %02hd:%02hd", localDate.GetHour(), localDate.GetMinute());
        }
    }
    return str;
}

SFXWideString AstronomyPane::GetMoonRiseString(Void) const
{
    SFXWideString str;
    AstronomicalMath::Date date;
    SFXDate localDate;

    if (_moonAge < 0.0) {
        // Not calculated yet
        str = "Moon Rise --:--";
    }
    else {
        if (_moon.GetRiseDateSize() == 0) {
            str = "Moon Rise --:--";
        }
        else {
            date = _moon.GetRiseDate(0);
            localDate = date.GetSFXDate() + _timeDifference + SFXDateDuration(30);
            str = SFXWideString::Format("Moon Rise %02hd:%02hd", localDate.GetHour(), localDate.GetMinute());
        }
    }
    return str;
}

SFXWideString AstronomyPane::GetMoonSetString(Void) const
{
    SFXWideString str;
    AstronomicalMath::Date date;
    SFXDate localDate;

    if (_moonAge < 0.0) {
        // Not calculated yet
        str = "Moon Set  --:--";
    }
    else {
        if (_moon.GetSetDateSize() == 0) {
            str = "Moon Set  --:--";
        }
        else {
            date = _moon.GetSetDate(0);
            localDate = date.GetSFXDate() + _timeDifference + SFXDateDuration(30);
            str = SFXWideString::Format("Moon Set  %02hd:%02hd", localDate.GetHour(), localDate.GetMinute());
        }
    }
    return str;
}

HANDLER_IMPLEMENT_VOIDRENDER(AstronomyPane, OnContent, graphics)
{
    SFXRectangle world(GetContentWorld());
    SInt16Const fontHeight(graphics->GetFontHeight());
    SFXRectangle moonAgeWindow;
    SFXRectangle moonAgePictureRect;
    SFXRectangle sunRiseWindow, sunSetWindow;
    SFXRectangle moonRiseWindow, moonSetWindow;
    SInt16 moonAgeInt(0);
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBBitmapSmp bitmap, resBitmap;
    SFCError error(SFERR_NO_ERROR);
    SFXRGBColor rgb(ASTRONOMYPANE_MOONAGE_TRANSPARENCYCOLOR);
    RGBVAL rgbVal(0);
    NativeColor native(0);

    if (!GetInheritEnable()) {
        return;
    }

    graphics->ClearRectangle(world, SFXRGBColor(ASTRONOMYPANE_BACKGROUND_COLOR));
    world.Deflate(10, 10, 10, 10);

    // Display the Months old
    if (_moonAge >= 0.0) {
        moonAgeInt = static_cast<SInt16>(floor(_moonAge + 0.5));
        if (moonAgeInt > 29) {
            moonAgeInt = 0;
        }
    }
    if (moonAgeInt > 0) {
        // Display the picture of the moon
        bitmap = SFXGraphics::CreateBitmap(SFXSize(ASTRONOMYPANE_MOONAGE_WIDTH, ASTRONOMYPANE_MOONAGE_HEIGHT));
        if (bitmap != null) {
            resBitmap = shell->LoadResBitmap(ASTRONOMYPANE_MOONAGERESFILE, MOONAGE_01 + (moonAgeInt - 1));
            if (resBitmap != null) {
                error = bitmap->BltIn(0, 0, ASTRONOMYPANE_MOONAGE_WIDTH, ASTRONOMYPANE_MOONAGE_HEIGHT, resBitmap, 0, 0, AEE_RO_COPY);
                if (error == SFERR_NO_ERROR) {
                    // Set transparency color
                    rgbVal = rgb.AsRGBVAL();
                    native = bitmap->RGBToNative(rgbVal);
                    bitmap->SetTransparencyColor(native);

                    // The display position is calculated
                    moonAgePictureRect.SetSize(ASTRONOMYPANE_MOONAGE_WIDTH, ASTRONOMYPANE_MOONAGE_HEIGHT);
                    moonAgePictureRect.SetOrigin(world.GetX() + 50, world.GetY());
                    graphics->DrawBitmap(bitmap, moonAgePictureRect, AEE_RO_TRANSPARENT);
                }
            }
        }
    }
    moonAgeWindow.SetSize(200, fontHeight);
    moonAgeWindow.SetOrigin(world.GetX() + 8, world.GetY() + (ASTRONOMYPANE_MOONAGE_HEIGHT - fontHeight) / 2 -30);
    graphics->DrawText(GetMoonAgeString(), moonAgeWindow, SFXRGBColor(0, 0, 0, 0));

    // The movement time of the sun is displayed
    sunSetWindow.SetSize(world.GetWidth(), fontHeight);
    sunSetWindow.SnapRightBottom(world.GetRightBottom());
    graphics->DrawText(GetSunSetString(), sunSetWindow, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT);
    sunRiseWindow.SetSize(world.GetWidth(), fontHeight);
    sunRiseWindow.SnapLeftBottom(sunSetWindow.GetLeftTop());
    sunRiseWindow.Offset(0, -2);
    graphics->DrawText(GetSunRiseString(), sunRiseWindow, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT);

    // The movement time of the moon is displayed
    moonSetWindow.SetSize(world.GetWidth(), fontHeight);
    moonSetWindow.SnapLeft(world.GetLeft());
    moonSetWindow.SnapBottom(sunRiseWindow.GetTop());
    moonSetWindow.Offset(0, -4);
    graphics->DrawText(GetMoonSetString(), moonSetWindow, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT);
    moonRiseWindow.SetSize(world.GetWidth(), fontHeight);
    moonRiseWindow.SnapLeftBottom(moonSetWindow.GetLeftTop());
    graphics->DrawText(GetMoonRiseString(), moonRiseWindow, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT);
}

//
// Heavenly body information is updated (for internal call).
//
Void AstronomyPane::UpdateInformationInner(Void)
{
    SFXRectangle display(SGRSoftkeyToolbox::GetWindowPosition());
    SFXRectangle rect;
    SFXGraphicsPtr graphics(SFXGraphics::GetInstance());
    SGRSoftkeyWindowPtr softkey(null);
    SFRResponderPtr director;
    Bool front(true);

    // The calculated effect is displayed
    rect.SetSize(display.GetWidth() - 20, graphics->GetFontHeight() * 2);
    rect.SnapLeftTop(display.GetLeftTop());
    rect.Offset((display.GetWidth() - rect.GetWidth()) / 2, (display.GetHeight() - rect.GetHeight()) / 2);

    _dialog = ::new SFRMessageDialog(rect, "calculating...", "");
    if (!_dialog) {
        static_throw(SFERR_NO_MEMORY);
    }
    else {
        static_throw(*_dialog);
    }
    if (!static_try()) {
        ::delete _dialog;
        _dialog = null;
    }
    if (static_try()) {
        static_throw(_dialog->RegisterHandler(SFEVT_KEY, HANDLER_BEFORE, HANDLER_FUNCTION(OnNoButtonDialogKey)));
    }
    if (static_try()) {
        SGRSoftkeyWindow::Initialize(_dialog);
        static_throw(_dialog->RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_BEFORE, HANDLER_FUNCTION(OnNoButtonDialogSoftkey)));
    }
    softkey = SGRSoftkeyWindow::GetInstance();
    if (softkey && static_try()) {
        _dialog->SetReference(softkey->Bind(_dialog));
    }

    if ((director = GetDirector()) != null) {
        if (director->GetPrevious(TYPE_WILDCARD, ATTRIBUTE_WILDCARD, STATUS_VISIBLE, false) != null) {
            front = false;
        }
    }
    if (front) {
        SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
    }

    // Begin an actual calculation after 100 ms
    _infoTimer.Cancel();
    _infoTimer.Set(CALLBACK_FUNCTION(OnUpdateInformation));
    _infoTimer.Schedule(100);
    return;
}

//
// Information on the heavenly body is updated using the current time.
//
Void AstronomyPane::UpdateInformation(Void)
{
    _gmtDate = SFXDate::CurrentUTCDate();
    _autoUpdate = true;
    UpdateInformationInner();
}

//
// The heavenly body information on the specified time is calculated.
// utcDate : Time in UTC.
//
Void AstronomyPane::UpdateInformation(SFXDateConstRef utcDate)
{
    _gmtDate = utcDate;
    _autoUpdate = false;
    UpdateInformationInner();
}

CALLBACK_IMPLEMENT_SFXTIMER(AstronomyPane, OnUpdateInformation)
{
    Compute();
    return;
}

//
// Heavenly body information calculation.
//
Void AstronomyPane::Compute(Void)
{
    SFXDateConst localDate(_gmtDate + _timeDifference);
    SFXDateConst date0(localDate.GetYear(), localDate.GetMonth(), localDate.GetDay(), 0, 0, 0);
    AstronomicalMath::DateConst initialDate(date0 - _timeDifference);
    SFXDate nextUpdate, currentDate;
    SFXDateDuration sec;
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    SFRResponderPtr director;
    Bool front(true);

    _moonAge = AstronomicalMath::GetMoonAge(AstronomicalMath::Date(_gmtDate));
    _sun.Set(initialDate, _longitude, _latitude, _height);
    _sun.Compute();
    _moon.Set(initialDate, _longitude, _latitude, _height);
    _moon.Compute();

    if (_dialog) {
        if (_dialog->GetIndexForward() == 0) {
            SGRSoftkeyWindow::Terminate(_dialog);
            _dialog->DialogHandler();
            _dialog = null;
        }
        else {
            app->SetRemoveDialog(_dialog);
            _dialog = null;
        }
    }

    if ((director = GetDirector()) != null) {
        if (director->GetPrevious(TYPE_WILDCARD, ATTRIBUTE_WILDCARD, STATUS_VISIBLE, false) != null) {
            front = false;
        }
    }
    if (front) {
        SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
    }

    if (_autoUpdate) {
        currentDate = SFXDate::CurrentDate();
        nextUpdate.Set(currentDate.GetYear(), currentDate.GetMonth(), currentDate.GetDay(), currentDate.GetHour(), 0, 0);
        nextUpdate.AddHour(1);
        sec = nextUpdate - currentDate;
        _hourTimer.Set(CALLBACK_FUNCTION(OnHourlyUpdate));
        _hourTimer.Schedule((static_cast<SInt32>(sec.AsSInt64()) + ASTRONOMYPANE_UPDATE_DELAY) * 1000);
    }
    return;
}

//
// The calendar is updated every hour.
//
CALLBACK_IMPLEMENT_SFXTIMER(AstronomyPane, OnHourlyUpdate)
{
    UpdateInformation();
    return;
}

HANDLER_IMPLEMENT_VOIDRESUME(AstronomyPane, OnAppResume, environment)
{
    unused(environment);

    if (_autoUpdate) {
        UpdateInformation();
    }
    return;
}

HANDLER_IMPLEMENT_VOIDSUSPEND(AstronomyPane, OnAppSuspend, reason, info)
{
    unused(reason);
    unused(info);

    _hourTimer.Cancel();
    _infoTimer.Cancel();
    return;
}

//
// No event handler for the button of the dialog.
// The useless key event is thrown away.
//
HANDLER_IMPLEMENT_BOOLEVENT(AstronomyPane, OnNoButtonDialogKey, event)
{
    unused(event);
    return true;
}

//
// The useless soft key event is thrown away.
//
HANDLER_IMPLEMENT_VOIDEVENT(AstronomyPane, OnNoButtonDialogSoftkey, event)
{
    unused(event);
    return;
}
