//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      AstronomyPane.hpp
//

#ifndef __ASTRONOMYPANE_HPP
#define __ASTRONOMYPANE_HPP

#include <SophiaFramework.hpp>
#include "AstronomicalMath.hpp"

#define ASTRONOMYPANE_DEFAULT_LONGITUDE     139.741     // Default value of observation point longitude (Tokyo)
#define ASTRONOMYPANE_DEFAULT_LATITUDE      35.658      // Default value of observation point latitude (Tokyo)
#define ASTRONOMYPANE_DEFAULT_HEIGHT        0.0         // Default value of observation point height (m)

SFMTYPEDEFCLASS(AstronomyPane)

class AstronomyPane : public SFRPane {

SFMSEALCOPY(AstronomyPane)

private:

    SFXDate _gmtDate;                   // The displayed time
    Float64 _moonAge;                   // The age of moon
    SFRMessageDialogPtr _dialog;        // Dialog displayed while calculating
    SFXDateDuration _timeDifference;    // Time difference from GMT
    Float64 _longitude;                 // Observation point longitude (east +)
    Float64 _latitude;                  // Observation point latitude(north +)
    Float64 _height;                    // Observation point height (m)
    AstronomicalMath::SunRiseSunSet _sun;       // Sunset and sunrise date
    AstronomicalMath::MoonRiseMoonSet _moon;    // Moonset and moonrise date
    Bool _autoUpdate;
    SFXTimer _infoTimer;
    SFXTimer _hourTimer;

public:

    explicit AstronomyPane(SFRResponderPtr responder, SFXRectangleConstRef rect, SFXDateDurationConstRef timeDifference = SFXDateDuration(9 * 3600), Float64 longitude = ASTRONOMYPANE_DEFAULT_LONGITUDE, Float64 latitude = ASTRONOMYPANE_DEFAULT_LATITUDE, Float64 height = ASTRONOMYPANE_DEFAULT_HEIGHT);
    virtual ~AstronomyPane(Void);

    Void UpdateInformation(Void);
    Void UpdateInformation(SFXDateConstRef utcDate);
    CALLBACK_DECLARE_SFXTIMER(OnUpdateInformation)
    Void SetObserverPosition(Float64 longitude, Float64 latitude, Float64 height = ASTRONOMYPANE_DEFAULT_HEIGHT);
    Void SetTimeDifference(SFXDateDurationConstRef timeDifference);
    CALLBACK_DECLARE_SFXTIMER(OnHourlyUpdate)
    SFXDateDurationConstRef GetTimeDifference(Void) const;

private:

    SFXWideString GetMoonAgeString(Void) const;
    SFXWideString GetSunRiseString(Void) const;
    SFXWideString GetSunSetString(Void) const;
    SFXWideString GetMoonRiseString(Void) const;
    SFXWideString GetMoonSetString(Void) const;
    Void UpdateInformationInner(Void);
    Void Compute(Void);

    HANDLER_DECLARE_VOIDRENDER(OnContent)
    HANDLER_DECLARE_VOIDRESUME(OnAppResume)
    HANDLER_DECLARE_VOIDSUSPEND(OnAppSuspend)
    HANDLER_DECLARE_BOOLEVENT(OnNoButtonDialogKey)
    HANDLER_DECLARE_VOIDEVENT(OnNoButtonDialogSoftkey)
};

//
// The observation point location information is set.
//
inline Void AstronomyPane::SetObserverPosition(Float64 longitude, Float64 latitude, Float64 height)
{
    _longitude = longitude;
    _latitude = latitude;
    _height = height;
}

//
// The observation point time difference from GMT is set.
//
inline Void AstronomyPane::SetTimeDifference(SFXDateDurationConstRef timeDifference)
{
    _timeDifference = timeDifference;
}

//
// The observation point time difference from GMT is got.
//
inline SFXDateDurationConstRef AstronomyPane::GetTimeDifference(Void) const
{
    return _timeDifference;
}

#endif
