//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      CalendarPane.cpp
//

#include "CalendarPane.hpp"

CalendarPane::CalendarPane(SFRResponderPtr responder, SFXRectangleConstRef rect, SGRCalendarControl::YearFormatEnum yearFormat, SGRCalendarControl::MonthFormatEnum monthFormat, SGRCalendarControl::DayFormatEnum dayFormat, Bool beginMonday) static_throws : SFRPane(responder, rect, SFXMargin(0, 0, 0, 0)), _calendar(null)
{
    if (static_try()) {
        _calendar = ::new SGRCalendarControl(this, GetContentWorld(), yearFormat, monthFormat, dayFormat, beginMonday);
        if (_calendar == null) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_calendar);
        }
    }

    if (!static_try()) {
        ::delete _calendar;
        _calendar = null;
    }
    else {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_BEFORE, HANDLER_FUNCTION(OnKey)));
        _calendar->SetStatusTarget(true);
    }
    SetAutoUpdateTimer();
    return;
}

CalendarPane::~CalendarPane(Void)
{
    _timer.Cancel();
    return;
}

HANDLER_IMPLEMENT_BOOLEVENT(CalendarPane, OnKey, event)
{
    SFXDate cursor;
    SFXDate maxDate(0x7FFFFFFF);
    Bool result(false);

    switch (event.GetP16()) {
    case AVK_SOFT3:
        // Move to the previous month using the soft key 3
        cursor = _calendar->GetCursor();
        if (cursor >= SFXDate(1981, 2, 1, 0, 0, 0)) {
            cursor.SubMonth(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        result = true;
        break;
    case AVK_SOFT4:
        // Move to the next month using the soft keys 4
        cursor = _calendar->GetCursor();
        maxDate.Set(maxDate.GetYear(), 1, 1, 0, 0, 0);
        maxDate.SubMonth(1);
        if (cursor < maxDate) {
            cursor.AddMonth(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        result = true;
        break;
    case AVK_0:
    case AVK_1:
    case AVK_2:
    case AVK_3:
    case AVK_4:
    case AVK_5:
    case AVK_6:
    case AVK_7:
    case AVK_8:
    case AVK_9:
    case AVK_STAR:
    case AVK_POUND:
    case AVK_CLR:
        // 0-9 keys and the clear key "Move to today"
        _calendar->SetCursor(SFXDate::CurrentUTCDate() + _calendar->GetTimeDifference());
        _calendar->InvalidateContent();
        result = true;
        break;
    }

    return result;
}

//
// Automatic date update.
//
CALLBACK_IMPLEMENT_SFXTIMER(CalendarPane, AutoUpdate)
{
    SFRResponderPtr director;
    Bool front(true);

    if ((director = GetDirector()) != null) {
        if (director->GetPrevious(TYPE_WILDCARD, ATTRIBUTE_WILDCARD, STATUS_VISIBLE, false) != null) {
            front = false;
        }
    }
    if (front) {
        SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
    }
    SetAutoUpdateTimer();
    return;
}

//
// The calendar is updated at regular intervals (to take the correspondence of "Today").
//
Void CalendarPane::SetAutoUpdateTimer(Void)
{
    SFXDate currentDate, nextUpdate;
    SFXDateDuration sec;

    if (!_calendar) {
        return;
    }

    _timer.Cancel();
    currentDate = SFXDate::CurrentUTCDate() + _calendar->GetTimeDifference();
    nextUpdate.Set(currentDate.GetYear(), currentDate.GetMonth(), currentDate.GetDay(), currentDate.GetHour(), 0, 0);
    nextUpdate.AddHour(1);
    sec = nextUpdate - currentDate;
    _timer.Set(CALLBACK_FUNCTION(AutoUpdate));
    _timer.Schedule(static_cast<SInt32>(sec.AsSInt64()) * 1000);
    return;
}

HANDLER_IMPLEMENT_VOIDRESUME(CalendarPane, OnAppResume, environment)
{
    unused(environment);

    InvalidateContent();
    SetAutoUpdateTimer();
    return;
}

HANDLER_IMPLEMENT_VOIDSUSPEND(CalendarPane, OnAppSuspend, reason, info)
{
    unused(reason);
    unused(info);

    _timer.Cancel();
    return;
}

//
// The time difference is set for "Today" (all calculations with the UTC base).
//
Void CalendarPane::SetTimeDifference(SFXDateDurationConstRef difference)
{
    if (_calendar) {
        _calendar->SetTimeDifference(difference);
        SetAutoUpdateTimer();
    }
    return;
}
