//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      CalendarPane.hpp
//

#ifndef __CALENDARPANE_HPP
#define __CALENDARPANE_HPP

#include <SophiaFramework.hpp>
#include "SGRCalendarControl.hpp"

SFMTYPEDEFCLASS(CalendarPane)

class CalendarPane : public SFRPane {

SFMSEALCOPY(CalendarPane)

private:

    SGRCalendarControlPtr _calendar;
    SFXTimer _timer;

public:

    explicit CalendarPane(SFRResponderPtr responder, SFXRectangleConstRef rect, SGRCalendarControl::YearFormatEnum yearFormat = SGRCalendarControl::YEARFORMAT_AD, SGRCalendarControl::MonthFormatEnum monthFormat = SGRCalendarControl::MONTHFORMAT_ENGLISH, SGRCalendarControl::DayFormatEnum dayFormat = SGRCalendarControl::DAYFORMAT_ENGLISH, Bool beginMonday = false) static_throws;
    virtual ~CalendarPane(Void);

    Void SetTimeDifference(SFXDateDurationConstRef difference);
    SFXDateDuration GetTimeDifference(Void) const;
    Void SetCursor(SFXDateConstRef date);
    SFXDate GetCursor(Void) const;
    CALLBACK_DECLARE_SFXTIMER(AutoUpdate)
    HANDLER_DECLARE_BOOLEVENT(OnKey)
    HANDLER_DECLARE_VOIDRESUME(OnAppResume)
    HANDLER_DECLARE_VOIDSUSPEND(OnAppSuspend)

private:

    Void SetAutoUpdateTimer(Void);
};

inline SFXDateDuration CalendarPane::GetTimeDifference(Void) const
{
    return _calendar ? _calendar->GetTimeDifference() : SFXDateDuration(9 * 3600);
}

inline Void CalendarPane::SetCursor(SFXDateConstRef date)
{
    if (_calendar) {
        _calendar->SetCursor(date);
    }
}

inline SFXDate CalendarPane::GetCursor(Void) const
{
    SFXDate cursor(_calendar ? _calendar->GetCursor() : SFXDate::CurrentDate());
    SFXDate result(cursor.GetYear(), cursor.GetMonth(), cursor.GetDay(), 0, 0, 0);
    return result;
}

#endif
