//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      CityAdditionDialog.cpp
//

#include "CityAdditionDialog.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"

#define CITYADDITIONDIALOG_LABEL_SOFTKEY1   "Register"
#define CITYADDITIONDIALOG_LABEL_SOFTKEY2   "Cancel"
#define CITYADDITIONDIALOG_LABEL_SOFTKEY3   "Select"

CityAdditionDialog::CityAdditionDialog(SFXWideStringConstRef name, SFXWideStringConstRef timeDifference, SFXWideStringConstRef longitude, SFXWideStringConstRef latitude, SFXWideStringConstRef height) static_throws : SFRTitleDialog(SGRSoftkeyToolbox::GetWindowPosition(), "City addition"), _nameControl(null), _timeDifferenceControl(null), _latitudeControl(null), _longitudeControl(null), _heightControl(null)
{
    SFXRectangle world(GetContentWorld());
    SGRSoftkeyWindowPtr softkey;

    SGRSoftkeyWindow::Initialize(this);

    // Coordinates calculation
    world.Deflate(10, 10, 10, 10);
    _nameLabelRect.Set(world);
    _nameLabelRect.DivHeight(10);
    _nameControlRect.Set(_nameLabelRect);
    _nameControlRect.Offset(0, _nameLabelRect.GetHeight());
    _timeDifferenceLabelRect.Set(_nameControlRect);
    _timeDifferenceLabelRect.Offset(0, _nameControlRect.GetHeight());
    _timeDifferenceControlRect.Set(_timeDifferenceLabelRect);
    _timeDifferenceControlRect.Offset(0, _timeDifferenceLabelRect.GetHeight());
    _latitudeLabelRect.Set(_timeDifferenceControlRect);
    _latitudeLabelRect.Offset(0, _timeDifferenceControlRect.GetHeight());
    _latitudeControlRect.Set(_latitudeLabelRect);
    _latitudeControlRect.Offset(0, _latitudeLabelRect.GetHeight());
    _longitudeLabelRect.Set(_latitudeControlRect);
    _longitudeLabelRect.Offset(0, _latitudeControlRect.GetHeight());
    _longitudeControlRect.Set(_longitudeLabelRect);
    _longitudeControlRect.Offset(0, _longitudeLabelRect.GetHeight());
    _heightLabelRect.Set(_longitudeControlRect);
    _heightLabelRect.Offset(0, _longitudeControlRect.GetHeight());
    _heightControlRect.Set(_heightLabelRect);
    _heightControlRect.Offset(0, _heightLabelRect.GetHeight());

    _nameControlRect.Deflate(10, 0, 30, 0);
    _timeDifferenceControlRect.Deflate(10, 0, 30, 0);
    _latitudeControlRect.Deflate(10, 0, 30, 0);
    _longitudeControlRect.Deflate(10, 0, 30, 0);
    _heightControlRect.Deflate(10, 0, 30, 0);


    // Responder creation
    if (static_try()) {
        _nameControl = ::new SFREditboxControl(this, _nameControlRect, name);
        if (!_nameControl) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_nameControl);
        }
        if (!static_try()) {
            ::delete _nameControl;
            _nameControl = null;
        }
        else {
            _nameControl->SetMaxSize(20);
        }
    }
    if (static_try()) {
        _timeDifferenceControl = ::new SFREditboxControl(this, _timeDifferenceControlRect, timeDifference);
        if (!_timeDifferenceControl) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_timeDifferenceControl);
        }
        if (!static_try()) {
            ::delete _timeDifferenceControl;
            _timeDifferenceControl = null;
        }
        else {
            _timeDifferenceControl->SetMaxSize(5);
        }
    }
    if (static_try()) {
        _latitudeControl = ::new SFREditboxControl(this, _latitudeControlRect, latitude);
        if (!_latitudeControl) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_latitudeControl);
        }
        if (!static_try()) {
            ::delete _latitudeControl;
            _latitudeControl = null;
        }
        else {
            _latitudeControl->SetMaxSize(10);
        }
    }
    if (static_try()) {
        _longitudeControl = ::new SFREditboxControl(this, _longitudeControlRect, longitude);
        if (!_longitudeControl) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_longitudeControl);
        }
        if (!static_try()) {
            ::delete _longitudeControl;
            _longitudeControl = null;
        }
        else {
            _longitudeControl->SetMaxSize(10);
        }
    }
    if (static_try()) {
        _heightControl = ::new SFREditboxControl(this, _heightControlRect, height);
        if (!_heightControl) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_heightControl);
        }
        if (!static_try()) {
            ::delete _heightControl;
            _heightControl = null;
        }
        else {
            _heightControl->SetMaxSize(10);
        }
    }

    // Handler registration
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
    if (static_try()) {
        static_throw(SFRToolbox::RegisterManipulateHandler(this));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, AVK_SELECT, HANDLER_AFTER, HANDLER_FUNCTION(OnSelect)));
    }
    if (static_try()) {
        static_throw(_nameControl->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }
    if (static_try()) {
        static_throw(_timeDifferenceControl->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }
    if (static_try()) {
        static_throw(_latitudeControl->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }
    if (static_try()) {
        static_throw(_longitudeControl->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }
    if (static_try()) {
        static_throw(_heightControl->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditbox)));
    }

    // Softkey registration
    if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) {
        static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_1, CITYADDITIONDIALOG_LABEL_SOFTKEY1));
        if (static_try()) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_2, CITYADDITIONDIALOG_LABEL_SOFTKEY2));
        }
        if (static_try()) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, CITYADDITIONDIALOG_LABEL_SOFTKEY3));
        }
        if (static_try()) {
            SetReference(softkey->Bind(this));
        }
    }

    if (_nameControl) {
        _nameControl->SetStatusFocus(true);
    }
}

CityAdditionDialog::~CityAdditionDialog(Void)
{
    SGRSoftkeyWindow::Terminate(this);
}

HANDLER_IMPLEMENT_VOIDRENDER(CityAdditionDialog, OnContent, graphics)
{
    SFRTitleDialog::ContentHandler(graphics);
    graphics->DrawText("City", _nameLabelRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Time difference (Example : +09:30  +0930)", _timeDifferenceLabelRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Latitude (North +, South -)", _latitudeLabelRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Longitude (East +, West -)", _longitudeLabelRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText("Height (m) (Usually Setting with 0)", _heightLabelRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
}

HANDLER_IMPLEMENT_BOOLVOID(CityAdditionDialog, OnSelect)
{
    SFRControlPtr control;
    Bool result(false);

     if ((control = static_cast<SFRControlPtr>(GetFocus())) != null) {
        if (control->GetAttribute() == ATTRIBUTE_SFREDITBOXCONTROL) {
            control->SetStatusTarget(true);
#if defined TARGET_ENVIRONMENT_PHYSICAL
            result = control->Invoke(SFXEvent(SFEVT_KEY, AVK_SELECT, 0));
#else
            result = true;
#endif
        }
    }

    return result;
}

//
// Text control input end handler.
//
HANDLER_IMPLEMENT_VOIDCONTROL(CityAdditionDialog, OnEditbox, result, control)
{
    unused(result);

    control->SetStatusTarget(false);
    return;
}

//
// Dialog information is acquired (with data check function).
//
CityAdditionDialog::NewCityInformation CityAdditionDialog::GetNewCityInformation(Void) const
{
    NewCityInformation info;
    SInt32 d(0), hour(0), minute(0);
    Float64 f(0.0);
    Bool valid(true), minus(false);

    // The string is acquired from the control
    if (_nameControl) {
        info._name.Set(_nameControl->GetText().Trim());
    }
    if (_timeDifferenceControl) {
        info._timeDifference.Set(_timeDifferenceControl->GetText().Trim());
    }
    if (_longitudeControl) {
        info._longitude.Set(_longitudeControl->GetText().Trim());
    }
    if (_latitudeControl) {
        info._latitude.Set(_latitudeControl->GetText().Trim());
    }
    if (_heightControl) {
        info._height.Set(_heightControl->GetText().Trim());
    }

    // The data validity is checked
    info._errorMessage.Clear();
    info._invalidControl = null;
    if (info._name.IsEmptyCString()) {
        info._errorMessage.Set("The city name is not entered.");
        info._invalidControl = _nameControl;
        valid = false;
    }
    if (valid) {
        d = info._timeDifference.AsSInt32();
        if (d < 0) {
            d = - d;
        }
        hour = d / 100;
        minute = d % 100;
        if (hour >= 12 || minute >= 60) {
            info._errorMessage.Set("The value of the time difference is illegal.");
            info._invalidControl = _timeDifferenceControl;
            valid = false;
        }
    }
    if (valid) {
        f = SFXHelper::wstrtofloat(info._longitude.GetCString());
        if (f < 0.0) {
            f = - f;
            minus = true;
        }
        else {
            minus = false;
        }
        if (f >= 180.0) {
            info._errorMessage.Set("The value of the longitude is illegal. ");
            info._invalidControl = _longitudeControl;
            valid = false;
        }
        else {
            d = static_cast<SInt32>(floor(f * 1000.0 + 0.5));
            if (minus) {
                info._height.Set(SFXWideString::Format("-%d.%d", d / 1000, d % 1000));
            }
            else {
                info._height.Set(SFXWideString::Format("%d.%d", d / 1000, d % 1000));
            }
        }
    }
    if (valid) {
        f = SFXHelper::wstrtofloat(info._latitude.GetCString());
        if (f < 0.0) {
            f = - f;
            minus = true;
        }
        else {
            minus = false;
        }
        if (f >= 90.0) {
            info._errorMessage.Set("The value of latitude is illegal. ");
            info._invalidControl = _latitudeControl;
            valid = false;
        }
        else {
            d = static_cast<SInt32>(floor(f * 1000.0 + 0.5));
            if (minus) {
                info._height.Set(SFXWideString::Format("-%d.%d", d / 1000, d % 1000));
            }
            else {
                info._height.Set(SFXWideString::Format("%d.%d", d / 1000, d % 1000));
            }
        }
    }
    if (valid) {
        f = SFXHelper::wstrtofloat(info._height.GetCString());
        if (f < 0.0) {
            f = - f;
            minus = true;
        }
        else {
            minus = false;
        }
        d = static_cast<SInt32>(floor(f * 1000.0 + 0.5));
        if (minus) {
            info._height.Set(SFXWideString::Format("-%d.%d", d / 1000, d % 1000));
        }
        else {
            info._height.Set(SFXWideString::Format("%d.%d", d / 1000, d % 1000));
        }
    }

    return info;
}
