//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      CityInformation.hpp
//

#ifndef __CITYINFORMATION_HPP
#define __CITYINFORMATION_HPP

#include <SophiaFramework.hpp>
#include "Array.hpp"

#define CITYINFORMATION_FILENAME    "cityinfo.txt"      // File that stores city information

SFMTYPEDEFCLASS(CityInformation)
class CityInformation {

SFMSEALCOPY(CityInformation)

private:

    SFMTYPEDEFSTRUCT(CITYINNER)
    struct CITYINNER {
        SFXAnsiString _name;                // The city name
        SFXDateDuration _timeDifference;    // Difference in time
        SFXAnsiString _longitude;           // Longitude
        SFXAnsiString _latitude;            // Latitude
        SFXAnsiString _height;              // Height
    };
    Array<CITYINNER> _array;
    Bool _changed;

public:

    SFMTYPEDEFSTRUCT(CITY)
    struct CITY {
        SFXWideString _name;                // The city name
        SFXDateDuration _timeDifference;    // Difference in time
        Float64 _longitude;                 // Longitude
        Float64 _latitude;                  // Latitude
        Float64 _height;                    // Height
    };

public:

    CityInformation(Void);
    virtual ~CityInformation(Void);
    SInt32 GetSize(Void) const;
    CITY Get(SInt32 index) const;
    SFCError Append(SFXWideStringConstRef cityName, SFXWideStringConstRef timeDifference, SFXWideStringConstRef longitude, SFXWideStringConstRef latitude, SFXWideStringConstRef height);
    SFCError Append(SFXWideStringConstRef cityName, SFXWideStringConstRef timeDifference, SFXWideStringConstRef longitude, SFXWideStringConstRef latitude, SFXWideStringConstRef height, BoolPtr appended, SInt32Ptr index);
    Void Remove(SInt32 index);
    SInt32 Find(SFXWideStringConstRef cityName) const;

private:

    static Void ReadLine(SFXAnsiStringPtr string, SFXBinaryStreamReaderPtr reader);
    static Bool ParseText(CITYINNERPtr city, SFXAnsiStringConstRef string);
};

inline SInt32 CityInformation::GetSize(Void) const
{
    return _array.GetSize();
}

inline Void CityInformation::Remove(SInt32 index)
{
    _array.Remove(index);
    _changed = true;
}

#endif
