//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      CitySelectionDialog.cpp
//

#include "CitySelectionDialog.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"
#include "pclock.hpp"
#include "CityInformation.hpp"

#define CITYSELECTIONDIALOG_LABEL_SOFTKEY1          "Add"
#define CITYSELECTIONDIALOG_LABEL_SOFTKEY2          "Delete"
#define CITYSELECTIONDIALOG_LABEL_SOFTKEY3          "Select"
#define CITYSELECTIONDIALOG_ERRORDIALOG_HEIGHT      220
#define CITYSELECTIONDIALOG_REMOVEDIALOG_HEIGHT     220

CitySelectionDialog::CitySelectionDialog(Void) static_throws : SFRTitleDialog(SGRSoftkeyToolbox::GetWindowPosition(), "City selection"), _listBox(null), _additionDialog(null), _gpsDialog(null), _invalidTextControl(null)
{
    SInt16Const fontHeight(SFXGraphics::GetFontHeight(AEE_FONT_NORMAL));
    SFXRectangleConst displayRect(GetContentWorld());
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    CityInformationPtr cityInfo(app->GetCityInformationPointer());
    CityInformation::CITY city;
    SInt32 i;
    SGRSoftkeyWindowPtr softkey;

    SGRSoftkeyWindow::Initialize(this);
    _textRect.SetSize(displayRect.GetWidth(), fontHeight * 2);
    _textRect.SetOrigin(displayRect.GetOrigin());
    _listBoxRect = displayRect;
    _listBoxRect.Deflate(20, _textRect.GetHeight(), 20, 10);
    _textRect.Deflate(10, 0, 10, 0);

    _listBox = ::new SGRListboxControl(this, _listBoxRect);
    if (!_listBox) {
        static_throw(SFERR_NO_MEMORY);
    }
    else {
        static_throw(*_listBox);
    }
    if (!static_try()) {
        ::delete _listBox;
        _listBox = null;
    }
    else {
        for (i = 0; i < cityInfo->GetSize(); ++i) {
            city = cityInfo->Get(i);
            static_throw(_listBox->Append(city._name));
            if (!static_try()) {
                break;
            }
        }
    }

    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
    if (static_try()) {     // Resumed
        static_throw(RegisterHandler(SFEVT_APP_RESUME, HANDLER_BEFORE, HANDLER_FUNCTION(OnResume)));
    }
    if (static_try()) {     // Suspended
        static_throw(RegisterHandler(SFEVT_APP_SUSPEND, HANDLER_AFTER, HANDLER_FUNCTION(OnSuspend)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
    }
    if (static_try()) {
        if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_1, CITYSELECTIONDIALOG_LABEL_SOFTKEY1));
            if (static_try()) {
                static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_2, CITYSELECTIONDIALOG_LABEL_SOFTKEY2));
            }
            if (static_try()) {
                static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, CITYSELECTIONDIALOG_LABEL_SOFTKEY3));
            }
            if (static_try()) {
                SetReference(softkey->Bind(this));
            }
        }
    }

    if (_listBox) {
        _listBox->SetSelect(static_cast<SInt16>(cityInfo->Find(app->GetCurrentCity())));
        _listBox->SetStatusTarget(true);
    }

    SFXHelper::memset(&_gpsInfo, 0, sizeof(_gpsInfo));
    _gpsCallback.Set(CALLBACK_FUNCTION(OnGPSNotify));
}

CitySelectionDialog::~CitySelectionDialog(Void)
{
    _gpsCallback.Cancel();
    SGRSoftkeyWindow::Terminate(this);
}

HANDLER_IMPLEMENT_VOIDRENDER(CitySelectionDialog, OnContent, graphics)
{
    SFRTitleDialog::ContentHandler(graphics);
    graphics->DrawText("Please select the city :", _textRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
}

HANDLER_IMPLEMENT_VOIDEVENT(CitySelectionDialog, OnSoftkey, event)
{
    SInt32 result(SUCCESS), index(0);
    SFXRectangle display, rect;
    SFRMultiDialogPtr dialog;
    SFRMultiDialog::ParamRec param;
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    CityInformationPtr cityInfo(app->GetCityInformationPointer());
    CityInformation::CITY city;
    SGRSoftkeyWindowPtr softkey(null);

    switch (event.GetP16()) {
    case SGRSoftkeyWindow::SOFTKEY_1:
        // Addition of a new city
        // The location information acquiring effect is displayed
        display.Set(SGRSoftkeyToolbox::GetWindowPosition());
        rect.SetSize(display.GetWidth() - 20, SFXGraphics::GetFontHeight(AEE_FONT_NORMAL) * 2);
        rect.SnapLeftTop(display.GetLeftTop());
        rect.Offset((display.GetWidth() - rect.GetWidth()) / 2, (display.GetHeight() - rect.GetHeight()) / 2);
        _gpsDialog = ::new SFRMessageDialog(rect, "Getting the location information...", "");
        if (!_gpsDialog) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_gpsDialog);
        }
        if (!static_try()) {
            ::delete _gpsDialog;
            _gpsDialog = null;
        }
        else {
            static_throw(_gpsDialog->RegisterHandler(SFEVT_KEY, HANDLER_BEFORE, HANDLER_FUNCTION(OnNoButtonDialogKey)));
        }
        if (static_try()) {
            SGRSoftkeyWindow::Initialize(_gpsDialog);
            static_throw(_gpsDialog->RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_BEFORE, HANDLER_FUNCTION(OnNoButtonDialogSoftkey)));
        }
        softkey = SGRSoftkeyWindow::GetInstance();
        if (softkey && static_try()) {
            _gpsDialog->SetReference(softkey->Bind(_gpsDialog));
        }
        if (_gpsDialog) {
            SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
        }
        // Start GPS
        _posDet = SFBPosDet::NewInstance();
        if (_posDet == null) {
            OnGPS(false);
        }
        else {
            SFXHelper::memset(&_gpsInfo, 0, sizeof(_gpsInfo));
            _gpsCallback.Set(CALLBACK_FUNCTION(OnGPSNotify));
            result = _posDet->GetGPSInfo(AEEGPS_GETINFO_LOCATION | AEEGPS_GETINFO_ALTITUDE, AEEGPS_ACCURACY_LEVEL6, &_gpsInfo, &_gpsCallback);
            if (result != SUCCESS) {
                // Start GPS failing
                _gpsCallback.Cancel();
                _posDet.Release();
                OnGPS(false);
            }
        }
        break;
    case SGRSoftkeyWindow::SOFTKEY_2:
        // City information deletion
        if (cityInfo->GetSize() <= 1) {
            // One or more is necessary
            param.text = "At least one is necessary for city information.";
            param.ok = "OK";
            param.cancel = "";
            param.another = "";
            param.focus = SFRMultiDialog::FOCUS_OK;
            param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
            dialog = ::new SFRMultiDialog(SGRSoftkeyToolbox::GetDialogPosition(SFRMultiDialog::GetMargin(), CITYSELECTIONDIALOG_ERRORDIALOG_HEIGHT), "Error", param);
            if (!dialog) {
                static_throw(SFERR_NO_MEMORY);
            }
            else {
                static_throw(*dialog);
            }
            if (!static_try()) {
                ::delete dialog;
                dialog = null;
            }
            else {
                SGRSoftkeyToolbox::OpenStandardDialog(dialog, HANDLER_FUNCTION(OnRemoveErrorDialog));
            }
        }
        else {
            // Deletion confirmation
            if (_listBox) {
                index = _listBox->GetSelect();
                city = cityInfo->Get(index);
                param.text = "\"" + city._name + "\" will be deleted. Is it ok?";
                param.ok = "Yes";
                param.cancel = "No";
                param.another = "";
                param.focus = SFRMultiDialog::FOCUS_CANCEL;
                param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, ACCEPT);
                dialog = ::new SFRMultiDialog(SGRSoftkeyToolbox::GetDialogPosition(SFRMultiDialog::GetMargin(), CITYSELECTIONDIALOG_REMOVEDIALOG_HEIGHT), "Confirmation", param);
                if (!dialog) {
                    static_throw(SFERR_NO_MEMORY);
                }
                else {
                    static_throw(*dialog);
                }
                if (!static_try()) {
                    ::delete dialog;
                    dialog = null;
                }
                else {
                    SGRSoftkeyToolbox::OpenStandardDialog(dialog, HANDLER_FUNCTION(OnRemoveDialog));
                }
            }
        }
        break;
    }
}

CALLBACK_IMPLEMENT_SFXCALLBACK(CitySelectionDialog, OnGPSNotify)
{
    _posDet.Release();
    OnGPS(true);
    return;
}

//
// Processing after GPS ends.
// If valid = true, the dialog is initialized using the GPS information.
//
Void CitySelectionDialog::OnGPS(Bool valid)
{
    Float64 longitude_f, latitude_f;
    SInt32 val, height_i;
    SFXWideString longitude_s, latitude_s, height_s;
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));

    if (_gpsDialog) {
        if (_gpsDialog->GetIndexForward() == 0) {
            SGRSoftkeyWindow::Terminate(_gpsDialog);
            _gpsDialog->DialogHandler();
            _gpsDialog = null;
        }
        else {
            app->SetRemoveDialog(_gpsDialog);
            _gpsDialog = null;
        }
    }

    if (valid) {
        // GPS data analysis
        if (_gpsInfo.fValid & AEEGPS_VALID_LON) {
            longitude_f = SFBPosDet::WGS84ToDegrees(_gpsInfo.dwLon);
        }
        else {
            longitude_f = 0.0;
        }
        if (_gpsInfo.fValid & AEEGPS_VALID_LAT) {
            latitude_f = SFBPosDet::WGS84ToDegrees(_gpsInfo.dwLat);
        }
        else {
            latitude_f = 0.0;
        }
        if (_gpsInfo.fValid & AEEGPS_VALID_ALT) {
            height_i = _gpsInfo.wAltitude;
        }
        else {
            height_i = 0;
        }
    }
    else {
        longitude_f = 0.0;
        latitude_f = 0.0;
        height_i = 0;
    }
    if (longitude_f < 0.0) {
        val = static_cast<SInt32>(floor((- longitude_f) * 1000.0 + 0.5));
        longitude_s = SFXWideString::Format("-%d.%d", val / 1000, val % 1000);
    }
    else {
        val = static_cast<SInt32>(floor(longitude_f * 1000.0 + 0.5));
        longitude_s = SFXWideString::Format("+%d.%d", val / 1000, val % 1000);
    }
    if (latitude_f < 0.0) {
        val = static_cast<SInt32>(floor((- latitude_f) * 1000.0 + 0.5));
        latitude_s = SFXWideString::Format("-%d.%d", val / 1000, val % 1000);
    }
    else {
        val = static_cast<SInt32>(floor(latitude_f * 1000.0 + 0.5));
        latitude_s = SFXWideString::Format("+%d.%d", val / 1000, val % 1000);
    }
    height_s = SFXWideString::Format("%d.0", height_i);

    // The city information registration dialog is displayed
    _additionDialog = ::new CityAdditionDialog("Present place", "+0900", longitude_s, latitude_s, height_s);
    if (!_additionDialog) {
        static_throw(SFERR_NO_MEMORY);
    }
    else {
        static_throw(*_additionDialog);
    }
    if (!static_try()) {
        ::delete _additionDialog;
        _additionDialog = null;
    }

    // The city information registration dialog softkey event handler is registered
    if (static_try()) {
        static_throw(_additionDialog->RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnAdditionDialogSoftKey)));
    }
}

HANDLER_IMPLEMENT_VOIDEVENT(CitySelectionDialog, OnAdditionDialogSoftKey, event)
{
    CityAdditionDialog::NewCityInformation cityInfo;
    SFRMultiDialogPtr errorDialog(null);
    SFRMultiDialog::ParamRec param;
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    CityInformationPtr appCityInfo(app->GetCityInformationPointer());
    Bool appended(false);
    SInt32 index(0);

    _invalidTextControl = null;
    switch (event.GetP16()) {
    case SGRSoftkeyWindow::SOFTKEY_1:
        // Register
        if (_additionDialog) {
            cityInfo = _additionDialog->GetNewCityInformation();
            if (!cityInfo._errorMessage.IsEmptyCString()) {
                // An illegal value was entered
                _invalidTextControl = cityInfo._invalidControl;
                param.text = cityInfo._errorMessage;
                param.ok = "Ok";
                param.cancel = "";
                param.another = "";
                param.focus = SFRMultiDialog::FOCUS_OK;
                param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
                errorDialog = ::new SFRMultiDialog(SGRSoftkeyToolbox::GetDialogPosition(SFRMultiDialog::GetMargin(), CITYSELECTIONDIALOG_ERRORDIALOG_HEIGHT), "G[", param);
                if (!errorDialog) {
                    static_throw(SFERR_NO_MEMORY);
                }
                else {
                    static_throw(*errorDialog);
                }
                if (static_try()) {
                    SGRSoftkeyToolbox::OpenStandardDialog(errorDialog, HANDLER_FUNCTION(OnErrorDialog));
                }
                else {
                    ::delete errorDialog;
                    errorDialog = null;
                }
            }
            else {
                // A correct value was entered
                _additionDialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
                _additionDialog = null;
                if (_listBox) {
                    _listBox->SetStatusTarget(true);
                }
                if (appCityInfo->Append(cityInfo._name, cityInfo._timeDifference, cityInfo._longitude, cityInfo._latitude, cityInfo._height, &appended, &index) != SFERR_NO_ERROR) {
                    // Can't register due to lack of memory
                    param.text = "  It failed in registration.";
                    param.ok = "Ok";
                    param.cancel = "";
                    param.another = "";
                    param.focus = SFRMultiDialog::FOCUS_OK;
                    param.image = SFBShell::GetInstance()->LoadResImage(PCLOCK_RESOURCE_FILENAME, CANCEL);
                    errorDialog = ::new SFRMultiDialog(SGRSoftkeyToolbox::GetDialogPosition(SFRMultiDialog::GetMargin(), CITYSELECTIONDIALOG_ERRORDIALOG_HEIGHT), "G[", param);
                    if (!errorDialog) {
                        static_throw(SFERR_NO_MEMORY);
                    }
                    else {
                        static_throw(*errorDialog);
                    }
                    if (static_try()) {
                        SGRSoftkeyToolbox::OpenStandardDialog(errorDialog, HANDLER_FUNCTION(OnErrorDialog));
                    }
                    else {
                        ::delete errorDialog;
                        errorDialog = null;
                    }
                }
                else {
                    if (appended) {
                        // New registration
                        static_throw(_listBox->Append(cityInfo._name));
                    }
                    _listBox->SetSelect(static_cast<SInt16>(index));
                }
            }
        }
        break;
    case SGRSoftkeyWindow::SOFTKEY_2:
        // Cancel
        if (_additionDialog) {
            _additionDialog->Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        }
        break;
    }
}

HANDLER_IMPLEMENT_VOIDDIALOG(CitySelectionDialog, OnErrorDialog, result, dialog)
{
    unused(result);

    SGRSoftkeyToolbox::CloseStandardDialog(dialog);
    if (_invalidTextControl) {
        _invalidTextControl->SetStatusFocus(true);
    }
    else if (_listBox) {
        _listBox->SetStatusTarget(true);
    }
}

HANDLER_IMPLEMENT_VOIDDIALOG(CitySelectionDialog, OnRemoveErrorDialog, result, dialog)
{
    unused(result);

    SGRSoftkeyToolbox::CloseStandardDialog(dialog);
    if (_listBox) {
        _listBox->SetStatusTarget(true);
    }
}

HANDLER_IMPLEMENT_VOIDDIALOG(CitySelectionDialog, OnRemoveDialog, result, dialog)
{
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));
    CityInformationPtr cityInfo(app->GetCityInformationPointer());
    SInt32 index(0);

    switch (result) {
    case SRP16_OK:
        SGRSoftkeyToolbox::CloseStandardDialog(dialog);
        if (_listBox) {
            index = _listBox->GetSelect();
            cityInfo->Remove(index);
            _listBox->Remove(static_cast<SInt16>(index));
            _listBox->SetStatusTarget(true);
        }
        break;
    case SRP16_ESCAPE:
    case SRP16_CANCEL:
        SGRSoftkeyToolbox::CloseStandardDialog(dialog);
        if (_listBox) {
            _listBox->SetStatusTarget(true);
        }
        break;
    }
}

//
// Dialog event handler.
// The useless key event is thrown away.
//
HANDLER_IMPLEMENT_BOOLEVENT(CitySelectionDialog, OnNoButtonDialogKey, event)
{
    unused(event);
    return true;
}

//
// The useless softkey event is thrown away.
//
HANDLER_IMPLEMENT_VOIDEVENT(CitySelectionDialog, OnNoButtonDialogSoftkey, event)
{
    unused(event);
}
HANDLER_IMPLEMENT_VOIDRESUME(CitySelectionDialog, OnResume, environment)
{
    unused(environment);

    SInt32 result(SUCCESS);

    if (_gpsDialog) {
        SFXHelper::memset(&_gpsInfo, 0, sizeof(_gpsInfo));
        _gpsCallback.Set(CALLBACK_FUNCTION(OnGPSNotify));
        result = _posDet->GetGPSInfo(AEEGPS_GETINFO_LOCATION | AEEGPS_GETINFO_ALTITUDE, AEEGPS_ACCURACY_LEVEL6, &_gpsInfo, &_gpsCallback);
        if (result != SUCCESS) {
            // It fails starting the GPS
            _gpsCallback.Cancel();
            _posDet.Release();
            OnGPS(false);
        }
    }
}

HANDLER_IMPLEMENT_VOIDSUSPEND(CitySelectionDialog, OnSuspend, reason, info)
{
    unused(reason);
    unused(info);

    // When suspending, GPS is ended
    _gpsCallback.Cancel();
    _posDet.Release();
}
