//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      ClockPane.cpp
//

#include "ClockPane.hpp"
#include "pclock.hpp"

#define CLOCKPANE_UPDATE_INTERVAL           200     // Screen is updated every 200 msec
#define CLOCKPANE_UPDATE_INTERVAL2          1000    // Update frequency when ClockWindow is not displayed on the foreground
#define CLOCKPANE_HOURHAND_TO_MINUTEHAND    0.7     // (Hour hand/seconds hand)
#define CLOCKPANE_HAND_COLOR                153, 153, 153, 0    // Colors of minute hand and hour hand
#define CLOCKPANE_SECONDHAND_COLOR          204, 204, 204, 0    // Colors of seconds hand
#define CLOCKPANE_BACKGROUND_COLOR          255, 255, 255, 0    // Background color

ClockPane::ClockPane(SFRResponderPtr responder, SFXRectangleConstRef rect, SFXWideStringConstRef cityName, SFXDateDurationConstRef timeDifference) static_throws : SFRPane(responder, rect, SFXMargin(0, 0, 0, 0)), _timeDifference(timeDifference), _cityName(cityName), _simulation(false), _lastDate(0)
{
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_APP_SUSPEND, HANDLER_AFTER, HANDLER_FUNCTION(OnAppSuspend)));
    }
    return;
}

ClockPane::~ClockPane(Void)
{
    _timer.Cancel();
    return;
}

HANDLER_IMPLEMENT_VOIDRENDER(ClockPane, OnContent, graphics)
{
    SFXRectangle world(GetContentWorld());
    SInt16Const fontHeight(graphics->GetFontHeight());
    SFXRectangle clockRect, alarmRect, analogClockRect, rect;
    SFXDate localDate;
    SInt16 handCenterX, handCenterY, handX, handY;
    Float64 handLength, secondHandLength, fHour, fMinute, fSecond, theta;
    SFXWideString alarmString;
    pclockPtr app(static_cast<pclockPtr>(SFRApplication::GetInstance()));

    if (!GetInheritEnable()) {
        return;
    }

    graphics->ClearRectangle(world, SFXRGBColor(CLOCKPANE_BACKGROUND_COLOR));

    alarmRect.SetSize(world.GetWidth(), fontHeight + 10);
    clockRect.SetSize(world.GetWidth(), world.GetHeight() - alarmRect.GetHeight());
    clockRect.SetOrigin(world.GetOrigin());
    alarmRect.SetOrigin(world.GetOrigin());
    alarmRect.SnapTop(clockRect.GetBottom());
    alarmRect.Inflate(2, 0, 2, 0);
    analogClockRect.SetSize(clockRect.GetHeight(), clockRect.GetHeight());
    analogClockRect.SnapLeftBottom(clockRect.GetLeftBottom());
    clockRect.Deflate(10, 10, 10, 10);

    localDate = (_simulation ? _simulationDate : SFXDate::CurrentUTCDate()) + _timeDifference;

    if (_lastDate != localDate) {
        // Draw an analog clock
        handCenterX = analogClockRect.GetWidth() / 2 + analogClockRect.GetLeft();
        handCenterY = analogClockRect.GetHeight() / 2 + analogClockRect.GetTop();
        handLength = static_cast<Float64>(((analogClockRect.GetWidth() < analogClockRect.GetHeight()) ? analogClockRect.GetWidth() : analogClockRect.GetHeight()) / 2 - 2);
        secondHandLength = handLength * CLOCKPANE_HOURHAND_TO_MINUTEHAND;
        fHour = static_cast<Float64>(localDate.GetHour());
        fMinute = static_cast<Float64>(localDate.GetMinute());
        fSecond = static_cast<Float64>(localDate.GetSecond());
        // Hour hand coordinates calculation
        theta = DegreeToRadian(- 90.0 + 30.0 * fHour + fMinute / 2.0 + fSecond / 120.0);
        handX = static_cast<SInt16>(secondHandLength * cos(theta)) + handCenterX;
        handY = static_cast<SInt16>(secondHandLength * sin(theta)) + handCenterY;
        _hourHand.Set(handCenterX, handCenterY, handX, handY);
        // Minute hand coordinates calculation
        theta = DegreeToRadian(- 90.0 + 6.0 * fMinute + fSecond / 10.0);
        handX = static_cast<SInt16>(handLength * cos(theta)) + handCenterX;
        handY = static_cast<SInt16>(handLength * sin(theta)) + handCenterY;
        _minuteHand.Set(handCenterX, handCenterY, handX, handY);
        // Seconds hand coordinates calculation
        theta = DegreeToRadian(- 90.0 + 6.0 * fSecond);
        handX = static_cast<SInt16>(handLength * cos(theta)) + handCenterX;
        handY = static_cast<SInt16>(handLength * sin(theta)) + handCenterY;
        _secondHand.Set(handCenterX, handCenterY, handX, handY);
        _lastDate = localDate;
    }

    // Drawing processing
    graphics->DrawLine(_hourHand, SFXRGBColor(CLOCKPANE_HAND_COLOR));
    graphics->DrawLine(_minuteHand, SFXRGBColor(CLOCKPANE_HAND_COLOR));
    graphics->DrawLine(_secondHand, SFXRGBColor(CLOCKPANE_SECONDHAND_COLOR));

    graphics->DrawRectangle(alarmRect, SFXRGBColor(120, 120, 120, 0));
    alarmString = "ALARM - " + app->GetAlarmDate(_timeDifference);
    graphics->DrawText(alarmString, alarmRect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);

    rect = clockRect;
    graphics->DrawText(_cityName, rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_TOP);

    rect.SetHeight(fontHeight * 4 + 15);
    rect.Offset(0, (clockRect.GetHeight() - rect.GetHeight()) / 2);
    graphics->DrawText(SFXWideString::Format("%hd Year", localDate.GetYear()), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT | IDF_ALIGN_TOP);
    rect.Offset(0, fontHeight + 5);
    graphics->DrawText(SFXWideString::Format("%02hd Month", localDate.GetMonth()), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT | IDF_ALIGN_TOP);
    rect.Offset(0, fontHeight + 5);
    graphics->DrawText(SFXWideString::Format("%02hd Date", localDate.GetDay()), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT | IDF_ALIGN_TOP);
    rect.Offset(0, fontHeight + 5);
    graphics->DrawText(SFXWideString::Format("%02hd:%02hd:%02hd", localDate.GetHour(), localDate.GetMinute(), localDate.GetSecond()), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT | IDF_ALIGN_TOP);

    if (!_simulation) {
        _timer.Set(CALLBACK_FUNCTION(OnUpdate));
        _timer.Schedule((GetDirector()->GetIndexForward() == 0) ? CLOCKPANE_UPDATE_INTERVAL : CLOCKPANE_UPDATE_INTERVAL2);
    }
    return;
}

//
// The renewal of the clock is delayed.
// mSec : Time to delay the update (msec).
//
Void ClockPane::DelayUpdate(SInt32 mSec)
{
    _timer.Cancel();
    _timer.Set(CALLBACK_FUNCTION(OnUpdate));
    _timer.Schedule(mSec);
    return;
}

//
// Update screen.
//
CALLBACK_IMPLEMENT_SFXTIMER(ClockPane, OnUpdate)
{
    SFRResponderPtr director;
    Bool front(true);

    if ((director = GetDirector()) != null) {
        if (director->GetPrevious(TYPE_WILDCARD, ATTRIBUTE_WILDCARD, STATUS_VISIBLE, false) != null) {
            front = false;
        }
    }
    if (front) {
        SFRApplication::GetInstance()->Invoke(SFXEvent(SREVT_RESPONDER_RENDER, SRP16_RENDER_INVOKE, true));
    }
    return;
}

HANDLER_IMPLEMENT_VOIDSUSPEND(ClockPane, OnAppSuspend, reason, info)
{
    unused(reason);
    unused(info);

    _timer.Cancel();
    return;
}

//
// Switch to the simulation display.
// utcDate : Simulated date (UTC).
//
Void ClockPane::SetSimulation(SFXDateConstRef utcDate)
{
    _simulationDate = utcDate;
    _simulation = true;
    _timer.Cancel();
    InvalidateContent();
    return;
}

//
// Release simulation.
//
Void ClockPane::CancelSimulation(Void)
{
    _simulation = false;
    InvalidateContent();
    return;
}
