//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      DateSelectionDialog.cpp
//

#include "DateSelectionDialog.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"

#define DATESELECTIONDIALOG_HEIGHT 160
#define DATESELECTIONDIALOG_LABEL_SOFTKEY1  ""
#define DATESELECTIONDIALOG_LABEL_SOFTKEY2  ""
#define DATESELECTIONDIALOG_LABEL_SOFTKEY3  "Select"

DateSelectionDialog::DateSelectionDialog(SFXDateConstRef date, SFXDateDurationConstRef timeDifference, SGRCalendarControl::YearFormatEnum yearFormat, SGRCalendarControl::MonthFormatEnum monthFormat, SGRCalendarControl::DayFormatEnum dayFormat, Bool beginMonday) static_throws : SFRTitleDialog(ComputeWindowRectangle(), "Date selection"), _calendar(null)
{
    SGRSoftkeyWindowPtr softkey;

    SGRSoftkeyWindow::Initialize(this);

    if (static_try()) {
        _calendar = ::new SGRCalendarControl(this, GetContentWorld(), yearFormat, monthFormat, dayFormat, beginMonday);
        if (!_calendar) {
            static_throw(SFERR_NO_MEMORY);
        }
        else {
            static_throw(*_calendar);
        }
    }
    if (!static_try()) {
        ::delete _calendar;
        _calendar = null;
    }
    else {
        _calendar->SetCursor(date);
        _calendar->SetTimeDifference(timeDifference);
    }

    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_BEFORE, HANDLER_FUNCTION(OnKey)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
    }
    if (static_try()) {
        if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) {
            static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_1, DATESELECTIONDIALOG_LABEL_SOFTKEY1));
            if (static_try()) {
                static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_2, DATESELECTIONDIALOG_LABEL_SOFTKEY2));
            }
            if (static_try()) {
                static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, DATESELECTIONDIALOG_LABEL_SOFTKEY3));
            }
            if (static_try()) {
                SetReference(softkey->Bind(this));
            }
        }
    }

    if (_calendar) {
        _calendar->SetStatusTarget(true);
    }
}

DateSelectionDialog::~DateSelectionDialog(Void)
{
    SGRSoftkeyWindow::Terminate(this);
}

//
// The dialog (window) displayed position is calculated.
//
SFXRectangle DateSelectionDialog::ComputeWindowRectangle(Void)
{
    SFXRectangleConst display(SGRSoftkeyToolbox::GetWindowPosition());
    SFXRectangle rect;

    rect.SetOrigin(display.GetOrigin());
    rect.SetSize(display.GetWidth(), DATESELECTIONDIALOG_HEIGHT);
    rect.Offset((display.GetWidth() - rect.GetWidth()) / 2, (display.GetHeight() - rect.GetHeight()) / 2);

    return rect;
}

HANDLER_IMPLEMENT_BOOLEVENT(DateSelectionDialog, OnKey, event)
{
    SFXDate cursor;
    SFXDate maxDate(0x7FFFFFFF);
    Bool result(false);

    switch (event.GetP16()) {
    case AVK_SELECT:
        Invoke(SFXEvent(SREVT_WINDOW, SRP16_OK, reinterpret_cast<UInt32>(this)));
        result = true;
        break;
    case AVK_SOFT3:
        // It moves one year back using the softkey 3
        cursor = _calendar->GetCursor();
        if (cursor >= SFXDate(1982, 1, 1, 0, 0, 0)) {
            cursor.SubYear(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        result = true;
        break;
    case AVK_SOFT4:
        // It moves one year forward using the softkey 4
        cursor = _calendar->GetCursor();
        maxDate.Set(maxDate.GetYear(), 1, 1, 0, 0, 0);
        maxDate.SubYear(1);
        if (cursor < maxDate) {
            cursor.AddYear(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        result = true;
        break;
    case AVK_0:
    case AVK_1:
    case AVK_2:
    case AVK_3:
    case AVK_4:
    case AVK_5:
    case AVK_6:
    case AVK_7:
    case AVK_8:
    case AVK_9:
    case AVK_STAR:
    case AVK_POUND:
    case AVK_CLR:
        // the 0-9 keys and the clear key "Move to today. "
        _calendar->SetCursor(SFXDate::CurrentUTCDate() + _calendar->GetTimeDifference());
        _calendar->InvalidateContent();
        result = true;
        break;
    }

    return result;
}

HANDLER_IMPLEMENT_VOIDEVENT(DateSelectionDialog, OnSoftkey, event)
{
    SFXDate cursor;
    SFXDate maxDate(0x7FFFFFFF);

    switch (event.GetP16()) {
    case SGRSoftkeyWindow::SOFTKEY_1:
        // It moves one month back using the softkey 1
        cursor = _calendar->GetCursor();
        if (cursor >= SFXDate(1981, 2, 1, 0, 0, 0)) {
            cursor.SubMonth(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        break;
    case SGRSoftkeyWindow::SOFTKEY_2:
        // It moves one month forward using the softkey 2
        cursor = _calendar->GetCursor();
        maxDate.Set(maxDate.GetYear(), 1, 1, 0, 0, 0);
        maxDate.SubMonth(1);
        if (cursor < maxDate) {
            cursor.AddMonth(1);
        }
        _calendar->SetCursor(cursor);
        _calendar->InvalidateContent();
        break;
    case SGRSoftkeyWindow::SOFTKEY_3:
        _calendar->Invoke(SFXEvent(SFEVT_KEY, AVK_SOFT3, 0));
        break;
    }
}
