//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      SGRCalendarControl.cpp
//

#include "SGRCalendarControl.hpp"
#include "calendar_font_res.h"

#define SGRCALENDARCONTROL_NUMBERFONTRESFILE        "calendar_font.bar"
#define SGRCALENDARCONTROL_NUMBERFONTWIDTH          6
#define SGRCALENDARCONTROL_NUMBERFONTHEIGHT         11
#define SGRCALENDARCONTROL_DAYFONTRESFILE           "calendar_font.bar"
#define SGRCALENDARCONTROL_DAYFONTWIDTHENGLISH      14
#define SGRCALENDARCONTROL_DAYFONTWIDTHJAPANESE     11
#define SGRCALENDARCONTROL_DAYFONTHEIGHTENGLISH     7
#define SGRCALENDARCONTROL_DAYFONTHEIGHTJAPANESE    9
#define SGRCALENDARCONTROL_TODAYMARKRESFILE         "calendar_font.bar"
#define SGRCALENDARCONTROL_TODAYMARK_WIDTH          20
#define SGRCALENDARCONTROL_TODAYMARK_HEIGHT         20
#define SGRCALENDARCONTROL_BACKGROUND_COLOR         229, 242, 238, 0
#define SGRCALENDARCONTROL_CURSORRESFILE            "calendar_font.bar"
#define SGRCALENDARCONTROL_CURSOR_WIDTH             18
#define SGRCALENDARCONTROL_CURSOR_HEIGHT            18
#define SGRCALENDARCONTROL_TRANSPARENCYCOLOR        255, 0, 255, 0

//
// Coordinates in the left end of the figure bitmap are acquired.
//
inline SInt32 SGRCalendarControl::CalendarFont::GetNumberResourceLeft(UInt08 number)
{
    SInt32 result;

    switch (number) {
    case 0:
        result = 0;
        break;
    case 1:
        result = 7;
        break;
    case 2:
        result = 14;
        break;
    case 3:
        result = 21;
        break;
    case 4:
        result = 28;
        break;
    case 5:
        result = 35;
        break;
    case 6:
        result = 42;
        break;
    case 7:
        result = 49;
        break;
    case 8:
        result = 56;
        break;
    case 9:
        result = 63;
        break;
    default:
        result = 63;
    }
    return result;
}

//
// Coordinates of the top of the figure bitmap are acquired.
//
inline SInt32 SGRCalendarControl::CalendarFont::GetNumberResourceTop(UInt08 /* number */)
{
    return 0;
}

//
// Coordinates in the left end of the day of the week bitmap are acquired.
//
inline SInt32 SGRCalendarControl::CalendarFont::GetDayResourceLeft(SGRCalendarControl::CalendarFont::CalendarFontDayEnum day, SGRCalendarControl::DayFormatEnum format)
{
    SInt32 result;

    if (format == DAYFORMAT_JAPANESE) {
        switch (day) {
        case CALENDARFONT_DAY_SUNDAY:
            result = 0;
            break;
        case CALENDARFONT_DAY_MONDAY:
            result = 11;
            break;
        case CALENDARFONT_DAY_TUESDAY:
            result = 22;
            break;
        case CALENDARFONT_DAY_WEDNESDAY:
            result = 33;
            break;
        case CALENDARFONT_DAT_THURSDAY:
            result = 44;
            break;
        case CALENDARFONT_DAY_FRIDAY:
            result = 55;
            break;
        case CALENDARFONT_DAY_SATURDAY:
            result = 66;
            break;
        default:
            result = 0;
        }
    }
    else {
        switch (day) {
        case CALENDARFONT_DAY_SUNDAY:
            result = 0;
            break;
        case CALENDARFONT_DAY_MONDAY:
            result = 20;
            break;
        case CALENDARFONT_DAY_TUESDAY:
            result = 40;
            break;
        case CALENDARFONT_DAY_WEDNESDAY:
            result = 60;
            break;
        case CALENDARFONT_DAT_THURSDAY:
            result = 80;
            break;
        case CALENDARFONT_DAY_FRIDAY:
            result = 100;
            break;
        case CALENDARFONT_DAY_SATURDAY:
            result = 120;
            break;
        default:
            result = 0;
        }
    }

    return result;
}

//
// Y coordinates of the top of a day of the week.
//
inline SInt32 SGRCalendarControl::CalendarFont::GetDayResourceTop(SGRCalendarControl::CalendarFont::CalendarFontDayEnum /* day */, SGRCalendarControl::DayFormatEnum /* format */)
{
    return 0;
}

SGRCalendarControl::CalendarFont::CalendarFont(Void)
{
    Clear();
}

SGRCalendarControl::CalendarFont::CalendarFont(SGRCalendarControl::CalendarFontConstRef ob)
{
    Copy(ob);
}

SGRCalendarControl::CalendarFont::CalendarFont(UInt32 val, SGRCalendarControl::CalendarFont::CalendarFontNumberColorEnum color)
{
    Create(val, color);
}

SGRCalendarControl::CalendarFont::CalendarFont(SGRCalendarControl::CalendarFont::CalendarFontDayEnum day, SGRCalendarControl::DayFormatEnum format)
{
    Create(day, format);
}

SGRCalendarControl::CalendarFont::CalendarFont(SFXDate::DayOfWeekEnum day)
{
    Create(day);
}

SGRCalendarControl::CalendarFont::CalendarFont(SGRCalendarControl::CalendarFont::CalendarFontCursorEnum cursorType)
{
    Create(cursorType);
}

const SGRCalendarControl::CalendarFont &SGRCalendarControl::CalendarFont::operator=(SGRCalendarControl::CalendarFontConstRef ob)
{
    Copy(ob);
    return *this;
}

//
// The figure is converted.
//
Bool SGRCalendarControl::CalendarFont::Create(UInt32 val, SGRCalendarControl::CalendarFont::CalendarFontNumberColorEnum color)
{
    UInt32 d;
    SInt32 i, max;
    UInt08 number;
    SFXArray<UInt08> digit;
    SFXSize size;
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBBitmapSmp resBitmap;
    SFCError error;
    Bool result(true);

    // The figure of each digit is converted
    while (val > 0) {
        d = val % 10;
        digit.Append(static_cast<UInt08>(d));
        val /= 10;
    }
    max = digit.GetSize();
    size.Set(static_cast<SInt16>(SGRCALENDARCONTROL_NUMBERFONTWIDTH * max), SGRCALENDARCONTROL_NUMBERFONTHEIGHT);

    // The bitmap is constructed
    Clear();
    _bitmap = SFXGraphics::CreateBitmap(size);
    if (_bitmap == null) {
        return false;
    }
    switch (color) {
    case CALENDARFONT_COLOR_BLACK:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_NUMBERFONTRESFILE, CALENDARFONT_NUMBER_WEEKDAY);
        break;
    case CALENDARFONT_COLOR_BLUE:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_NUMBERFONTRESFILE, CALENDARFONT_NUMBER_SATURDAY);
        break;
    case CALENDARFONT_COLOR_RED:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_NUMBERFONTRESFILE, CALENDARFONT_NUMBER_SUNDAY);
        break;
    case CALENDARFONT_COLOR_WHITE:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_NUMBERFONTRESFILE, CALENDARFONT_NUMBER_TODAY);
        break;
    }
    if (resBitmap == null) {
        return false;
    }

    // Copy the bitmap
    for (i = 0; i < max; ++i) {
        number = digit.Get(max - i - 1);
        error = _bitmap->BltIn(SGRCALENDARCONTROL_NUMBERFONTWIDTH * i, 0, SGRCALENDARCONTROL_NUMBERFONTWIDTH, SGRCALENDARCONTROL_NUMBERFONTHEIGHT, resBitmap, GetNumberResourceLeft(number), GetNumberResourceTop(number), AEE_RO_COPY);
        if (error != SFERR_NO_ERROR) {
            result = false;
            Clear();
            break;
        }
    }
    if (result) {
        _size = size;
        SetTransparencyColor(SFXRGBColor(SGRCALENDARCONTROL_TRANSPARENCYCOLOR));
    }
    return result;
}

//
// The day of the week is converted.
//
Bool SGRCalendarControl::CalendarFont::Create(SGRCalendarControl::CalendarFont::CalendarFontDayEnum day, SGRCalendarControl::DayFormatEnum format)
{
    SFXSize size;
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBBitmapSmp resBitmap;
    SFCError error;

    // The bitmap is constructed
    if (format == DAYFORMAT_ENGLISH) {
        size.Set(SGRCALENDARCONTROL_DAYFONTWIDTHENGLISH, SGRCALENDARCONTROL_DAYFONTHEIGHTENGLISH);
    }
    else {
        size.Set(SGRCALENDARCONTROL_DAYFONTWIDTHJAPANESE, SGRCALENDARCONTROL_DAYFONTHEIGHTJAPANESE);
    }
    Clear();
    _bitmap = SFXGraphics::CreateBitmap(size);
    if (_bitmap == null) {
        return false;
    }
    if (format == DAYFORMAT_ENGLISH) {
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_DAYFONTRESFILE, CALENDARFONT_WEEK_ENGLISH);
    }
    else {
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_DAYFONTRESFILE, CALENDARFONT_WEEK_JAPANESE);
    }
    if (resBitmap == null) {
        return false;
    }

    // Copy the bitmap
    error = _bitmap->BltIn(0, 0, size.GetWidth(), size.GetHeight(), resBitmap, GetDayResourceLeft(day, format), GetDayResourceTop(day, format), AEE_RO_COPY);
    if (error == SFERR_NO_ERROR) {
        _size = size;
        SetTransparencyColor(SFXRGBColor(SGRCALENDARCONTROL_TRANSPARENCYCOLOR));
        return true;
    }
    else {
        Clear();
        return false;
    }
}

//
// "Today" Mark.
//
Bool SGRCalendarControl::CalendarFont::Create(SFXDate::DayOfWeekEnum day)
{
    SFXSizeConst size(SGRCALENDARCONTROL_TODAYMARK_WIDTH, SGRCALENDARCONTROL_TODAYMARK_HEIGHT);
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBBitmapSmp resBitmap;
    SFCError error;

    Clear();
    _bitmap = SFXGraphics::CreateBitmap(size);
    if (_bitmap == null) {
        return false;
    }
    switch (day) {
    case SFXDate::DAYOFWEEK_SUNDAY:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_TODAYMARKRESFILE, CALENDARMARK_TODAY_SUNDAY);
        break;
    case SFXDate::DAYOFWEEK_SATURDAY:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_TODAYMARKRESFILE, CALENDARMARK_TODAY_SATURDAY);
        break;
    default:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_TODAYMARKRESFILE, CALENDARMARK_TODAY_WEEKDAY);
    }
    if (resBitmap == null) {
        return false;
    }

    // Copy the bitmap
    error = _bitmap->BltIn(0, 0, size.GetWidth(), size.GetHeight(), resBitmap, 0, 0, AEE_RO_COPY);
    if (error == SFERR_NO_ERROR) {
        _size = size;
        SetTransparencyColor(SFXRGBColor(SGRCALENDARCONTROL_TRANSPARENCYCOLOR));
        return true;
    }
    else {
        Clear();
        return false;
    }
}

//
// Cursor.
//
Bool SGRCalendarControl::CalendarFont::Create(SGRCalendarControl::CalendarFont::CalendarFontCursorEnum cursorType)
{
    SFXSizeConst size(SGRCALENDARCONTROL_CURSOR_WIDTH, SGRCALENDARCONTROL_CURSOR_HEIGHT);
    SFBShellSmp shell(SFBShell::GetInstance());
    SFBBitmapSmp resBitmap;
    SFCError error;

    Clear();
    _bitmap = SFXGraphics::CreateBitmap(size);
    if (_bitmap == null) {
        return false;
    }
    switch (cursorType) {
    case CALENDARFONT_CURSOR_NORMAL:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_CURSORRESFILE, CALENDARCURSOR_NORMAL);
        break;
    case CALENDARFONT_CURSOR_TODAY:
        resBitmap = shell->LoadResBitmap(SGRCALENDARCONTROL_CURSORRESFILE, CALENDARCURSOR_TODAY);
        break;
    }
    if (resBitmap == null) {
        return false;
    }

    // Copy the bitmap
    error = _bitmap->BltIn(0, 0, size.GetWidth(), size.GetHeight(), resBitmap, 0, 0, AEE_RO_COPY);
    if (error == SFERR_NO_ERROR) {
        _size = size;
        SetTransparencyColor(SFXRGBColor(SGRCALENDARCONTROL_TRANSPARENCYCOLOR));
        return true;
    }
    else {
        Clear();
        return false;
    }
}

//
// Font information constructed as an image is annulled.
//
Void SGRCalendarControl::CalendarFont::Clear(Void)
{
    if (_bitmap != null) {
        _bitmap.Release();
    }
    _size.Set(0, 0);
}

//
// Copy font.
//
Bool SGRCalendarControl::CalendarFont::Copy(SGRCalendarControl::CalendarFontConstRef ob)
{
    SFCError error;

    Clear();
    _bitmap = SFXGraphics::CreateBitmap(ob._size);
    if (_bitmap == null) {
        return false;
    }
    error = _bitmap->BltIn(0, 0, ob._size.GetWidth(), ob._size.GetHeight(), ob._bitmap, 0, 0, AEE_RO_COPY);
    if (error == SFERR_NO_ERROR) {
        _size = ob._size;
        SetTransparencyColor(SFXRGBColor(SGRCALENDARCONTROL_TRANSPARENCYCOLOR));
        return true;
    }
    else {
        Clear();
        return false;
    }
}

//
// Collocate in the specified rectangular area.
//
SFXRectangle SGRCalendarControl::CalendarFont::GetRectangle(SFXRectangleConstRef rect, UInt32 align)
{
    SFXRectangle result(0, 0, _size);

    // Right and left collocate
    if (align & IDF_ALIGN_RIGHT) {
        result.SnapRight(rect.GetRight());
    }
    else if (align & IDF_ALIGN_CENTER) {
        result.SnapLeft(rect.GetLeft());
        result.Offset((rect.GetWidth() - result.GetWidth()) / 2, 0);
    }
    else {
        result.SnapLeft(rect.GetLeft());
    }

    // Top and bottom collocate
    if (align & IDF_ALIGN_BOTTOM) {
        result.SnapBottom(rect.GetBottom());
    }
    else if (align & IDF_ALIGN_MIDDLE) {
        result.SnapTop(rect.GetTop());
        result.Offset(0, (rect.GetHeight() - result.GetHeight()) / 2);
    }
    else {
        result.SnapTop(rect.GetTop());
    }

    return result;
}

//
// Get bitmap.
//
inline SFBBitmapSmpConstRef SGRCalendarControl::CalendarFont::GetBitmap(Void) const
{
    return _bitmap;
}

SGRCalendarControl::SGRCalendarControl(SFRResponderPtr responder, SFXRectangleConstRef rect, YearFormatEnum yearFormat, MonthFormatEnum monthFormat, DayFormatEnum dayFormat, bool beginMonday) static_throws
 : SFRControl(responder, rect, SFXMargin(2, 2, 2, 2), SFMRESPONDERFILTER(CALENDARCONTROL, BEHAVIOR_CALENDARCONTROL), ATTRIBUTE_CALENDARCONTROL), _yearFormat(yearFormat), _monthFormat(monthFormat), _dayFormat(dayFormat), _beginMonday(beginMonday),
 _cursorDate(SFXDate::CurrentDate()), _timeDifference(9 * 3600)
{
    if (static_try()) {
        static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_BASE, HANDLER_BEFORE, HANDLER_FUNCTION(OnBase)));
    }
    if (static_try()) {
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT, HANDLER_BEFORE, HANDLER_FUNCTION(OnContent)));
    }
}

SGRCalendarControl::~SGRCalendarControl(Void)
{
}

HANDLER_IMPLEMENT_BOOLEVENT(SGRCalendarControl, OnKey, event)
{
    SFXDateConst minDate(1981, 1, 1, 0, 0, 0);
    SFXDate date(0x7FFFFFFF);
    SFXDate maxDate(date.GetYear(), 1, 1, 0, 0, 0);
    SFXDateDurationConst delta(date - maxDate);
    Bool result(true);

    if (delta < SFXDateDuration(14 * 24 * 3600)) {
        maxDate.SubYear(1);
    }

    date = _cursorDate;
    switch (event.GetP16()) {
    case AVK_LEFT:
        // Move the cursor before a day by the left key
        date.SubDay(1);
        if (date >= minDate) {
            _cursorDate = date;
            InvalidateContent();
        }
        break;
    case AVK_RIGHT:
        // Move the cursor after a day by the right key
        date.AddDay(1);
        if (date < maxDate) {
            _cursorDate = date;
            InvalidateContent();
        }
        break;
    case AVK_UP:
        // Move the cursor before a week by the up key
        date.SubDay(7);
        if (date >= minDate) {
            _cursorDate = date;
            InvalidateContent();
        }
        break;
    case AVK_DOWN:
        // Move the cursor after a week by the down key
        date.AddDay(7);
        if (date < maxDate) {
            _cursorDate = date;
            InvalidateContent();
        }
        break;
    case AVK_SELECT:
        // Notify parents
        Invoke(SFXEvent(SREVT_CONTROL, SRP16_OK, reinterpret_cast<UInt32>(this)));
        break;
    case AVK_CLR:
        SetStatusTarget(false);
        Invoke(SFXEvent(SREVT_CONTROL, SRP16_ESCAPE, reinterpret_cast<UInt32>(this)));
        break;
    default:
        result = false;
    }

    return result;
}

HANDLER_IMPLEMENT_VOIDRENDER(SGRCalendarControl, OnBase, graphics)
{
    SFXRectangle rect(GetBaseWorld());
    SFXBevelColor frameRGB(SFXRGBColor(0xAA, 0xAA, 0xAA, 0x00), SFXRGBColor(0xFF, 0xFF, 0xFF, 0x00), SFXRGBColor(0xFF, 0xFF, 0xFF, 0x00));

    if (GetInheritEnable()) {
        graphics->DrawBevelRectangle(rect, frameRGB);
        rect.Deflate(1, 1);
        graphics->DrawRectangle(rect, SFXRGBColor(0x00, 0x00, 0x00, 0x00));
    }
    else {
        graphics->FillRectangle(rect, frameRGB.GetBase());
        rect.Deflate(1, 1);
        graphics->DrawRectangle(rect, SFXRGBColor(0x66, 0x66, 0x66, 0x00));
    }
}

HANDLER_IMPLEMENT_VOIDRENDER(SGRCalendarControl, OnContent, graphics)
{
    SFXRectangle worldRect(GetContentWorld());
    SFXRectangle rect, rect2, rect3, rectTmp;
    SFXDate now(SFXDate::CurrentUTCDate());
    now.Add(_timeDifference);
    SFXDateConst today(now.GetYear(), now.GetMonth(), now.GetDay(), 0, 0, 0);
    SFXDateConst firstDate(_cursorDate.GetYear(), _cursorDate.GetMonth(), 1, 0, 0, 0);
    SFXDateConst lastDate(_cursorDate.GetYear(), _cursorDate.GetMonth() + 1, 1, 0, 0, 0);
    SFXRGBColor bgColor(SGRCALENDARCONTROL_BACKGROUND_COLOR);
    SInt16 cellIndex, cellBias, i;
    UInt16 d;
    Bool error(true);
    CalendarFont calendarFont;

    if (!GetInheritEnable()) {
        bgColor.AddRGB(0x11);
    }
    graphics->SetBackColor(bgColor);
    graphics->ClearRectangle(worldRect);
    worldRect.Deflate(2, 2, 2, 2);

    // _cursorDate regularization
    now.Set(_cursorDate.GetYear(), _cursorDate.GetMonth(), _cursorDate.GetDay(), 0, 0, 0);
    _cursorDate = now;

    // Draw on the year and the month
    rect = worldRect;
    rect.SetHeight(graphics->GetFontHeight() + 1);
    graphics->DrawText(GetYearString(), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_LEFT | IDF_ALIGN_MIDDLE);
    graphics->DrawText(GetMonthString(), rect, SFXRGBColor(0, 0, 0, 0), IDF_ALIGN_RIGHT | IDF_ALIGN_MIDDLE);
    rectTmp = worldRect;
    rectTmp.SetHeight(worldRect.GetHeight() - rect.GetHeight());
    rectTmp.SnapBottom(worldRect.GetBottom());
    worldRect = rectTmp;

    // Coordinates calculation necessary to draw in calendar
    rect = worldRect;
    rect.SetHeight((_dayFormat == DAYFORMAT_JAPANESE) ? SGRCALENDARCONTROL_DAYFONTHEIGHTJAPANESE : SGRCALENDARCONTROL_DAYFONTHEIGHTENGLISH + 2);
    rect.DivWidth(7);
    rectTmp = worldRect;
    rectTmp.SetHeight(worldRect.GetHeight() - rect.GetHeight() - 1);
    rectTmp.SnapBottom(worldRect.GetBottom());
    worldRect = rectTmp;
    rect2 = worldRect;
    rect2.DivWidth(7);
    rect2.DivHeight(6);     // Maximum is six weeks

    // The serial number is assigned from 0 to 41 to each cell
    // It corresponds to which cell before the first day of the moon
    cellBias = static_cast<SInt16>(firstDate.GetDayOfWeek());
    if (_beginMonday) {
        --cellBias;
    }
    else {
        if (cellBias == 6) {
            cellBias = -1;
        }
    }

    // Display the mark when "Today" is on the present moon
    if (today >= firstDate && today < lastDate) {
        cellIndex = static_cast<SInt16>(today.GetDay()) + cellBias;
        rectTmp = rect2;
        rectTmp.SetOrigin(rect2.GetWidth() * (cellIndex % 7) + rect2.GetX(), rect2.GetHeight() * (cellIndex / 7) + rect2.GetY());
        if (calendarFont.Create(today.GetDayOfWeek())) {
            rect3 = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            graphics->DrawBitmap(calendarFont.GetBitmap(), rect3, AEE_RO_TRANSPARENT);
        }
    }

    // Drawing week
    rectTmp = rect;
    if (!_beginMonday) {
        // Start on Sunday
        for (i = CalendarFont::CALENDARFONT_DAY_SUNDAY; i < CalendarFont::CALENDARFONT_DAY_LIMIT; ++i) {
            if (calendarFont.Create(static_cast<CalendarFont::CalendarFontDayEnum>(i), _dayFormat)) {
                rect3 = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                graphics->DrawBitmap(calendarFont.GetBitmap(), rect3, AEE_RO_TRANSPARENT);
            }
            rectTmp.AddOrigin(rect.GetWidth(), 0);
        }
    }
    else {
        // Start on Monday
        for (i = CalendarFont::CALENDARFONT_DAY_MONDAY; i < CalendarFont::CALENDARFONT_DAY_LIMIT; ++i) {
            if (calendarFont.Create(static_cast<CalendarFont::CalendarFontDayEnum>(i), _dayFormat)) {
                rect3 = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                graphics->DrawBitmap(calendarFont.GetBitmap(), rect3, AEE_RO_TRANSPARENT);
            }
            rectTmp.AddOrigin(rect.GetWidth(), 0);
        }
        if (calendarFont.Create(CalendarFont::CALENDARFONT_DAY_SUNDAY, _dayFormat)) {
            rect3 = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            graphics->DrawBitmap(calendarFont.GetBitmap(), rect3, AEE_RO_TRANSPARENT);
        }
    }

    // Drawing day
    for (now = firstDate; now < lastDate; now.AddDay(1)) {
        d = now.GetDay();
        cellIndex = static_cast<SInt16>(d) + cellBias;
        rectTmp = rect2;
        rectTmp.SetOrigin(rect2.GetWidth() * (cellIndex % 7) + rect2.GetX(), rect2.GetHeight() * (cellIndex / 7) + rect2.GetY());
        if (now == today) {
            // "Today"
            if (calendarFont.Create(d, CalendarFont::CALENDARFONT_COLOR_WHITE)) {
                rect = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                graphics->DrawBitmap(calendarFont.GetBitmap(), rect, AEE_RO_TRANSPARENT);
            }
        }
        else {
            switch (now.GetDayOfWeek()) {
            case SFXDate::DAYOFWEEK_SUNDAY:
                error = calendarFont.Create(d, CalendarFont::CALENDARFONT_COLOR_RED);
                break;
            case SFXDate::DAYOFWEEK_SATURDAY:
                error = calendarFont.Create(d, CalendarFont::CALENDARFONT_COLOR_BLUE);
                break;
            default:
                error = calendarFont.Create(d, CalendarFont::CALENDARFONT_COLOR_BLACK);
            }
            if (error) {
                rect = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                graphics->DrawBitmap(calendarFont.GetBitmap(), rect, AEE_RO_TRANSPARENT);
            }

        }
        if (now == _cursorDate) {
            // Drawing cursor
            if (GetInheritEnable() && GetInheritFocus() && GetInheritTarget()) {
                if (now == today) {
                    error = calendarFont.Create(CalendarFont::CALENDARFONT_CURSOR_TODAY);
                }
                else {
                    error = calendarFont.Create(CalendarFont::CALENDARFONT_CURSOR_NORMAL);
                }
                if (error) {
                    rect = calendarFont.GetRectangle(rectTmp, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
                    graphics->DrawBitmap(calendarFont.GetBitmap(), rect, AEE_RO_TRANSPARENT);
                }
            }
        }
    }
}

//
// The year is converted into the string.
//
SFXWideString SGRCalendarControl::GetYearString(Void) const
{
    SFXWideString result;
    SInt32Const y(static_cast<SInt32>(_cursorDate.GetYear()));

    if (_yearFormat == YEARFORMAT_ERANAME) {
        if (y < 1989) {     // The Showa era
            result = SFXWideString::Format("Showa %d age", y - 1925);
        }
        else {  // The Heisei year
            result = SFXWideString::Format("Heisei %d age", y - 1988);
        }
    }
    else {
        if (_monthFormat != MONTHFORMAT_ENGLISH) {
            result = SFXWideString::Format("%d Year", y);
        }
        else {
            result = SFXWideString::Format("%d", y);
        }
    }

    return result;
}

//
// The month is converted into the string.
//
SFXWideString SGRCalendarControl::GetMonthString(Void) const
{
    SFXWideString result;
    SInt32 m(static_cast<SInt32>(_cursorDate.GetMonth()));

    switch (_monthFormat) {
    case MONTHFORMAT_NUMBER:
        result = SFXWideString::Format("%d Month", m);
        break;
    case MONTHFORMAT_JAPANESE:
        switch (m) {
        case 1:
            result = "r";
            break;
        case 2:
            result = "@";
            break;
        case 3:
            result = "퐶";
            break;
        case 4:
            result = "K";
            break;
        case 5:
            result = "H";
            break;
        case 6:
            result = "";
            break;
        case 7:
            result = "";
            break;
        case 8:
            result = "t";
            break;
        case 9:
            result = "";
            break;
        case 10:
            result = "_";
            break;
        case 11:
            result = "";
            break;
        case 12:
            result = "t";
            break;
        }
        break;
    default:
        switch (m) {
        case 1:
            result = "Jan";
            break;
        case 2:
            result = "Feb";
            break;
        case 3:
            result = "Mar";
            break;
        case 4:
            result = "Apr";
            break;
        case 5:
            result = "May";
            break;
        case 6:
            result = "Jun";
            break;
        case 7:
            result = "Jul";
            break;
        case 8:
            result = "Aug";
            break;
        case 9:
            result = "Sep";
            break;
        case 10:
            result = "Oct";
            break;
        case 11:
            result = "Nov";
            break;
        case 12:
            result = "Dec";
            break;
        }
    }

    return result;
}
