//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      SGRNumberTextControl.hpp
//

#ifndef __SGRNUMBERTEXTCONTROL_HPP
#define __SGRNUMBERTEXTCONTROL_HPP

#include <SophiaFramework.hpp>

SFMRESPONDERTYPE(NUMBERTEXTCONTROL, TYPE_SFRCONTROL)
SFMRESPONDERATTRIBUTE(NUMBERTEXTCONTROL, four_char_code('N', 'u', 'm', 't'))
SFMRESPONDERBEHAVIOR(NUMBERTEXTCONTROL, BEHAVIOR_SFRCONTROL, STATUS_VISIBLE | PROPERTY_DIRECT, PROPERTY_TRAVEL | PROPERTY_SCROLLABLE)

SFMTYPEDEFCLASS(SGRNumberTextControl)

class SGRNumberTextControl : public SFRControl {

    SFMSEALCOPY(SGRNumberTextControl)

private:

    SInt32 _digit;          // Number of digits
    SInt32 _minValue;       // Minimum value
    SInt32 _maxValue;       // Maximum value
    SInt32 _currentDigit;   // Input number of digits
    SInt32 _value;          // Input value

public:

    explicit SGRNumberTextControl(SFRResponderPtr responder, SFXRectangleConstRef rect, SInt32 digit, SInt32 min, SInt32 max, SInt32 defaultValue);
    virtual ~SGRNumberTextControl(Void);
    SInt32 GetValue(Void) const;
    SFXWideString GetString(Void) const;

private:

    Void ProcessNumberKey(SInt32 d);

    HANDLER_DECLARE_BOOLEVENT(OnKey)
    HANDLER_DECLARE_VOIDRENDER(OnBase)
    HANDLER_DECLARE_VOIDRENDER(OnContent)
};

inline SInt32 SGRNumberTextControl::GetValue(Void) const
{
    SInt32 val(_value);

    if (_value < _minValue) {
        val = _minValue;
    }
    else if (_value > _maxValue) {
        val = _maxValue;
    }

    return val;
}

inline SFXWideString SGRNumberTextControl::GetString(Void) const
{
    return SFXWideString::Format("%d", (_value < 0) ? _minValue : _value);
}

#endif
