//
//     Copyright (C) 2006 - 2007 SophiaCradle, Inc.
//     All rights reserved.
//
//      pclock.hpp
//

#ifndef __PCLOCK_HPP
#define __PCLOCK_HPP

#include <SophiaFramework.hpp>
#include "pclock.bid"
#include "pclock_res.h"
#include "ClockWindow.hpp"
#include "CityInformation.hpp"

#define PCLOCK_RESOURCE_FILENAME "pclock.bar"

SFMTYPEDEFCLASS(pclock)
class pclock : public SFRApplication {

    SFMSEALCOPY(pclock)

public:

    SFMTYPEDEFSTRUCT(AlarmInformation)
    struct AlarmInformation {
        UInt16 _userCode;       // Alarm user code (not used)
        SFXWideString _subject; // Alarm subject
        SFXDate _date;          // Date of alarm (UTC)
    };

private:

    ClockWindowPtr _clockWindow;
    Bool _alarmFlag;
    SFRDialogPtr _toRemoveDialog;
    CityInformation _cityInfo;
    SFXWideString _currentCity;
    AlarmInformation _alarmInfo;
    AEEMediaData _mediaData;
    SFBMediaQCPSmp _mediaQCP;
    SFXTimer _timer;

public:

    static SFCInvokerPtr Factory(Void);
    CityInformationPtr GetCityInformationPointer(Void);
    SFXWideStringConstRef GetCurrentCity(Void) const;
    Void SetCurrentCity(SInt32 index);
    AlarmInformationConstRef GetAlarm(Void) const;
    Bool SetAlarm(AlarmInformationConstRef info);
    Bool CancelAlarm(Void);
    Void TellAlarm(Void);
    SFXWideString GetAlarmDate(SFXDateDurationConstRef timeDifference);
    Void SetRemoveDialog(SFRDialogPtr dialog);
    Bool IsAlarmActive(Void) const;

private:

    explicit pclock(Void) static_throws;
    virtual ~pclock(Void);

    HANDLER_DECLARE_VOIDSTART(OnAppStart)
    HANDLER_DECLARE_VOIDSTOP(OnAppStop)
    HANDLER_DECLARE_VOIDRESUME(OnAppResume)
    HANDLER_DECLARE_VOIDSUSPEND(OnAppSuspend)
    HANDLER_DECLARE_BOOLEVENT(OnAlarm)
    HANDLER_DECLARE_VOIDDIALOG(OnAlarmDialog)
    CALLBACK_DECLARE_SFXTIMER(OnTimer)
};

inline CityInformationPtr pclock::GetCityInformationPointer(Void)
{
    return &_cityInfo;
}

inline SFXWideStringConstRef pclock::GetCurrentCity(Void) const
{
    return _currentCity;
}

inline pclock::AlarmInformationConstRef pclock::GetAlarm(Void) const
{
    return _alarmInfo;
}

inline Void pclock::SetRemoveDialog(SFRDialogPtr dialog)
{
    _toRemoveDialog = dialog;
}

inline Bool pclock::IsAlarmActive(Void) const
{
    return SFBShell::GetInstance()->AlarmsActive() || _timer.IsValid();
}

#endif // __PCLOCK_HPP //
