# Modified by SFMAKMODIFIER Ver 1.0
#============================================================================
#  Name:
#    $(TARGET).MAK
#
#  Description:
#    Makefile to build the $(TARGET) downloadable module.
#
#   The following nmake targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#     filename.o    - make object file
#
#   The above targets can be made with the following command:
#
#     nmake /f $(TARGET).mak [target]
#
#  Assumptions:
#    1. The environment variable ADSHOME is set to the root directory of the
#       arm tools.
#    2. The version of ADS is 1.2 or above.
#
#  Notes:
#    None.
#
#
#        Copyright  2000-2001 QUALCOMM Incorporated.
#               All Rights Reserved.
#            QUALCOMM Proprietary/GTDR
#
#----------------------------------------------------------------------------
#============================================================================
SF_HOME        =$(SFHOME)
ARM_HOME       =$(ARMHOME)
BREW_HOME      =$(BREWDIR)
TARGET         =pclock
OBJS           =AlarmSettingDialog.o AstronomicalMath.o AstronomyPane.o CalendarPane.o CityAdditionDialog.o CityInformation.o CitySelectionDialog.o ClockPane.o ClockWindow.o DateSelectionDialog.o pclock.o SGRCalendarControl.o SGRListboxControl.o SGRNumberTextControl.o SGRSoftkeyToolbox.o SGRSoftkeyWindow.o VDialog.o 
APP_INCLUDES   = -I "$(BREWDIR)\inc" -I "$(SF_HOME)\Include"

#-------------------------------------------------------------------------------
# Target file name and type definitions
#-------------------------------------------------------------------------------

EXETYPE    = elf                # Target image file format
MODULE     = mod                # Downloadable module extension

#-------------------------------------------------------------------------------
# Target compile time symbol definitions
#
# Tells the SDK source stuffs that we're building a dynamic app.
#-------------------------------------------------------------------------------

# DYNAPP          = -DDYNAMIC_APP
DYNAPP          = -DDYNAMIC_APP


#-------------------------------------------------------------------------------
# Software tool and environment definitions
#-------------------------------------------------------------------------------

AEESRCPATH = $(BREW_HOME)\src
AEEINCPATH = $(BREW_HOME)\inc

ARMBIN = $(ARM_HOME)\bin        # ARM ADS application directory
ARMINC = $(ARM_HOME)\include    # ARM ADS include file directory
ARMLIB = $(ARM_HOME)\lib        # ARM ADS library directory

ARMCC    = $(ARMBIN)\armcc      # ARM ADS ARM 32-bit inst. set ANSI CPP compiler
ARMCPP   = $(ARMBIN)\armcpp     # ARM ADS ARM 32-bit inst. set ANSI CPP compiler
LD       = $(ARMBIN)\armlink     # ARM ADS linker
HEXTOOL  = $(ARMBIN)\fromelf     # ARM ADS utility to create hex file from image

OBJ_CMD    = -o                 # Command line option to specify output filename

#-------------------------------------------------------------------------------
# Processor architecture options
#-------------------------------------------------------------------------------

CPU = -cpu ARM7TDMI             # ARM7TDMI target processor

#-------------------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
#-------------------------------------------------------------------------------

ROPI     = ropi                 # Read-Only(code) Position independence
INTERWRK = interwork            # Allow ARM-Thumb interworking

APCS = -apcs /$(ROPI)/$(INTERWRK)/norwpi

#-------------------------------------------------------------------------------
# Additional compile time error checking options
#-------------------------------------------------------------------------------

CHK = -fa                       # Check for data flow anomolies

#-------------------------------------------------------------------------------
# Compiler output options
#-------------------------------------------------------------------------------

OUT = -c                        # Object file output only

#-------------------------------------------------------------------------------
# Compiler/assembler debug options
#-------------------------------------------------------------------------------

# DBG = -g                        # Enable debug
DBG = -g-

#-------------------------------------------------------------------------------
# Compiler optimization options
#-------------------------------------------------------------------------------

# OPT = -Ospace -O2               # Full compiler optimization for space
OPT = -Otime -O2

#-------------------------------------------------------------------------------
# Compiler code generation options
#-------------------------------------------------------------------------------

END = -littleend                # Compile for little endian memory architecture
ZA  = -zo                       # LDR may only access 32-bit aligned addresses

CODE = $(END) $(ZA)


#-------------------------------------------------------------------------------
# Include file search path options
#-------------------------------------------------------------------------------

INC = -I. -I$(AEEINCPATH) $(APP_INCLUDES)


#-------------------------------------------------------------------------------
# Compiler pragma emulation options
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Linker options
#-------------------------------------------------------------------------------

LINK_CMD = -o                    #Command line option to specify output file
                                 #on linking

ROPILINK = -ropi                 #Link image as Read-Only Position Independent

LINK_ORDER = -first AEEMod_Load

#-------------------------------------------------------------------------------
# HEXTOOL options
#-------------------------------------------------------------------------------

BINFORMAT = -bin


#-------------------------------------------------------------------------------
# Compiler flag definitions
#-------------------------------------------------------------------------------

CFLAGS0 = $(OUT) $(DYNAPP) $(CPU) $(APCS) $(CODE) $(CHK) $(DBG)
CFLAGS  = $(CFLAGS0) $(OPT)

#-------------------------------------------------------------------------------
# Linker flag definitions
#-------------------------------------------------------------------------------

# the -entry flag is not really needed, but it keeps the linker from reporting
# warning L6305W (no entry point).  The address
# LFLAGS = $(ROPILINK) -rwpi -entry 0x8000#
LFLAGS = $(ROPILINK) -rwpi -entry AEEMod_Load#0x8000#


#----------------------------------------------------------------------------
# Default target
#----------------------------------------------------------------------------

all :  $(TARGET).$(MODULE)

#----------------------------------------------------------------------------
# Clean target
#----------------------------------------------------------------------------

# The object subdirectory, target image file, and target hex file are deleted.

clean :
        @echo ---------------------------------------------------------------
        @echo CLEAN
        -del /f $(OBJS)
        -del /f $(TARGET).$(EXETYPE)
        -del /f $(TARGET).$(MODULE)
        @echo ---------------------------------------------------------------

#============================================================================
#                           DEFAULT SUFFIX RULES
#============================================================================

# The following are the default suffix rules used to compile all objects that
# are not specifically included in one of the module specific rules defined
# in the next section.

# The following macros are used to specify the output object file, MSG_FILE
# symbol definition and input source file on the compile line in the rules
# defined below.

SRC_CPP_FILE = $(@F:.o=.cpp)      # Input source file specification
SRC_C_FILE   = $(@F:.o=.c)      # Input source file specification
OBJ_FILE = $(OBJ_CMD) $(@F)   # Output object file specification

.SUFFIXES :
.SUFFIXES : .o .dep .c .cpp

#--------------------------------------------------------------------------
# C code inference rules
#----------------------------------------------------------------------------

.cpp.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $(@F)
        $(ARMCPP) $(CFLAGS) $(INC) $(OBJ_FILE) $(SRC_CPP_FILE)
        @echo ---------------------------------------------------------------


{$(AEESRCPATH)}.c.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $(@F)
        $(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE) $(AEESRCPATH)\$(SRC_C_FILE)
        @echo ---------------------------------------------------------------


#===============================================================================
#                           MODULE SPECIFIC RULES
#===============================================================================

APP_OBJS =  $(OBJS) "$(SF_HOME)\Library\SophiaFrameworkBrew310RVCTB.a"

#----------------------------------------------------------------------------
# Lib file targets
#----------------------------------------------------------------------------

$(TARGET).$(MODULE) : $(TARGET).$(EXETYPE)
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(HEXTOOL)  $(TARGET).$(EXETYPE) $(BINFORMAT) $(TARGET).$(MODULE)

$(TARGET).$(EXETYPE) : $(APP_OBJS)
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(LD) $(LINK_CMD) $(TARGET).$(EXETYPE) $(LFLAGS) $(APP_OBJS) $(LINK_ORDER)

#----------------------------------------------------------------------------
# Applet Specific Rules
#----------------------------------------------------------------------------


# --------------------------------------------
# DEPENDENCY LIST, DO NOT EDIT BELOW THIS LINE
# --------------------------------------------


alarmsettingdialog.o:	alarmsettingdialog.cpp
alarmsettingdialog.o:	alarmsettingdialog.hpp
alarmsettingdialog.o:	sgrnumbertextcontrol.hpp
alarmsettingdialog.o:	dateselectiondialog.hpp
alarmsettingdialog.o:	sgrcalendarcontrol.hpp
alarmsettingdialog.o:	pclock.hpp
alarmsettingdialog.o:	pclock.bid
alarmsettingdialog.o:	pclock_res.h
alarmsettingdialog.o:	clockwindow.hpp
alarmsettingdialog.o:	clockpane.hpp
alarmsettingdialog.o:	astronomypane.hpp
alarmsettingdialog.o:	astronomicalmath.hpp
alarmsettingdialog.o:	array.hpp
alarmsettingdialog.o:	calendarpane.hpp
alarmsettingdialog.o:	sgrcalendarcontrol.hpp
alarmsettingdialog.o:	cityselectiondialog.hpp
alarmsettingdialog.o:	sgrlistboxcontrol.hpp
alarmsettingdialog.o:	cityadditiondialog.hpp
alarmsettingdialog.o:	cityinformation.hpp
alarmsettingdialog.o:	array.hpp
alarmsettingdialog.o:	sgrsoftkeywindow.hpp
alarmsettingdialog.o:	sgrsoftkeytoolbox.hpp
astronomicalmath.o:	astronomicalmath.cpp
astronomicalmath.o:	astronomicalmath.hpp
astronomicalmath.o:	array.hpp
astronomypane.o:	astronomypane.cpp
astronomypane.o:	astronomypane.hpp
astronomypane.o:	astronomicalmath.hpp
astronomypane.o:	array.hpp
astronomypane.o:	clockwindow.hpp
astronomypane.o:	clockpane.hpp
astronomypane.o:	astronomypane.hpp
astronomypane.o:	calendarpane.hpp
astronomypane.o:	sgrcalendarcontrol.hpp
astronomypane.o:	cityselectiondialog.hpp
astronomypane.o:	sgrlistboxcontrol.hpp
astronomypane.o:	cityadditiondialog.hpp
astronomypane.o:	sgrsoftkeywindow.hpp
astronomypane.o:	sgrsoftkeytoolbox.hpp
astronomypane.o:	moon_res.h
astronomypane.o:	pclock.hpp
astronomypane.o:	pclock.bid
astronomypane.o:	pclock_res.h
astronomypane.o:	clockwindow.hpp
astronomypane.o:	cityinformation.hpp
astronomypane.o:	array.hpp
calendarpane.o:	calendarpane.cpp
calendarpane.o:	calendarpane.hpp
calendarpane.o:	sgrcalendarcontrol.hpp
cityadditiondialog.o:	cityadditiondialog.cpp
cityadditiondialog.o:	cityadditiondialog.hpp
cityadditiondialog.o:	sgrsoftkeywindow.hpp
cityadditiondialog.o:	sgrsoftkeytoolbox.hpp
cityinformation.o:	cityinformation.cpp
cityinformation.o:	cityinformation.hpp
cityinformation.o:	array.hpp
cityselectiondialog.o:	cityselectiondialog.cpp
cityselectiondialog.o:	cityselectiondialog.hpp
cityselectiondialog.o:	sgrlistboxcontrol.hpp
cityselectiondialog.o:	cityadditiondialog.hpp
cityselectiondialog.o:	sgrsoftkeywindow.hpp
cityselectiondialog.o:	sgrsoftkeytoolbox.hpp
cityselectiondialog.o:	pclock.hpp
cityselectiondialog.o:	pclock.bid
cityselectiondialog.o:	pclock_res.h
cityselectiondialog.o:	clockwindow.hpp
cityselectiondialog.o:	clockpane.hpp
cityselectiondialog.o:	astronomypane.hpp
cityselectiondialog.o:	astronomicalmath.hpp
cityselectiondialog.o:	array.hpp
cityselectiondialog.o:	calendarpane.hpp
cityselectiondialog.o:	sgrcalendarcontrol.hpp
cityselectiondialog.o:	cityselectiondialog.hpp
cityselectiondialog.o:	cityinformation.hpp
cityselectiondialog.o:	array.hpp
cityselectiondialog.o:	cityinformation.hpp
clockpane.o:	clockpane.cpp
clockpane.o:	clockpane.hpp
clockpane.o:	pclock.hpp
clockpane.o:	pclock.bid
clockpane.o:	pclock_res.h
clockpane.o:	clockwindow.hpp
clockpane.o:	clockpane.hpp
clockpane.o:	astronomypane.hpp
clockpane.o:	astronomicalmath.hpp
clockpane.o:	array.hpp
clockpane.o:	calendarpane.hpp
clockpane.o:	sgrcalendarcontrol.hpp
clockpane.o:	cityselectiondialog.hpp
clockpane.o:	sgrlistboxcontrol.hpp
clockpane.o:	cityadditiondialog.hpp
clockpane.o:	cityinformation.hpp
clockpane.o:	array.hpp
clockwindow.o:	clockwindow.cpp
clockwindow.o:	clockwindow.hpp
clockwindow.o:	clockpane.hpp
clockwindow.o:	astronomypane.hpp
clockwindow.o:	astronomicalmath.hpp
clockwindow.o:	array.hpp
clockwindow.o:	calendarpane.hpp
clockwindow.o:	sgrcalendarcontrol.hpp
clockwindow.o:	cityselectiondialog.hpp
clockwindow.o:	sgrlistboxcontrol.hpp
clockwindow.o:	cityadditiondialog.hpp
clockwindow.o:	sgrsoftkeywindow.hpp
clockwindow.o:	sgrsoftkeytoolbox.hpp
clockwindow.o:	alarmsettingdialog.hpp
clockwindow.o:	sgrnumbertextcontrol.hpp
clockwindow.o:	dateselectiondialog.hpp
clockwindow.o:	sgrcalendarcontrol.hpp
clockwindow.o:	pclock.hpp
clockwindow.o:	pclock.bid
clockwindow.o:	pclock_res.h
clockwindow.o:	clockwindow.hpp
clockwindow.o:	cityinformation.hpp
clockwindow.o:	array.hpp
clockwindow.o:	vdialog.hpp
dateselectiondialog.o:	dateselectiondialog.cpp
dateselectiondialog.o:	dateselectiondialog.hpp
dateselectiondialog.o:	sgrcalendarcontrol.hpp
dateselectiondialog.o:	sgrsoftkeywindow.hpp
dateselectiondialog.o:	sgrsoftkeytoolbox.hpp
pclock.o:	pclock.cpp
pclock.o:	pclock.hpp
pclock.o:	pclock.bid
pclock.o:	pclock_res.h
pclock.o:	clockwindow.hpp
pclock.o:	clockpane.hpp
pclock.o:	astronomypane.hpp
pclock.o:	astronomicalmath.hpp
pclock.o:	array.hpp
pclock.o:	calendarpane.hpp
pclock.o:	sgrcalendarcontrol.hpp
pclock.o:	cityselectiondialog.hpp
pclock.o:	sgrlistboxcontrol.hpp
pclock.o:	cityadditiondialog.hpp
pclock.o:	cityinformation.hpp
pclock.o:	array.hpp
pclock.o:	sgrsoftkeywindow.hpp
pclock.o:	sgrsoftkeytoolbox.hpp
sgrcalendarcontrol.o:	sgrcalendarcontrol.cpp
sgrcalendarcontrol.o:	sgrcalendarcontrol.hpp
sgrcalendarcontrol.o:	calendar_font_res.h
sgrlistboxcontrol.o:	sgrlistboxcontrol.cpp
sgrlistboxcontrol.o:	sgrlistboxcontrol.hpp
sgrnumbertextcontrol.o:	sgrnumbertextcontrol.cpp
sgrnumbertextcontrol.o:	sgrnumbertextcontrol.hpp
sgrsoftkeytoolbox.o:	sgrsoftkeytoolbox.cpp
sgrsoftkeytoolbox.o:	sgrsoftkeytoolbox.hpp
sgrsoftkeytoolbox.o:	sgrsoftkeywindow.hpp
sgrsoftkeywindow.o:	sgrsoftkeywindow.cpp
sgrsoftkeywindow.o:	sgrsoftkeywindow.hpp
vdialog.o:	vdialog.cpp
vdialog.o:	vdialog.hpp
vdialog.o:	pclock_res.h
vdialog.o:	pclock.hpp
vdialog.o:	pclock.bid
vdialog.o:	pclock_res.h
vdialog.o:	clockwindow.hpp
vdialog.o:	clockpane.hpp
vdialog.o:	astronomypane.hpp
vdialog.o:	astronomicalmath.hpp
vdialog.o:	array.hpp
vdialog.o:	calendarpane.hpp
vdialog.o:	sgrcalendarcontrol.hpp
vdialog.o:	cityselectiondialog.hpp
vdialog.o:	sgrlistboxcontrol.hpp
vdialog.o:	cityadditiondialog.hpp
vdialog.o:	cityinformation.hpp
vdialog.o:	array.hpp
vdialog.o:	sgrsoftkeywindow.hpp
vdialog.o:	sgrsoftkeytoolbox.hpp
